# -*- coding: utf-8 -*-
from collective.geo.behaviour.interfaces import ICoordinates
from collective.taxonomy.interfaces import ITaxonomy
from cpskin.core.behaviors.indexview import ICpskinIndexViewSettings
from cpskin.core.browser.folderview import configure_folderviews
from cpskin.core.interfaces import ICPSkinCoreLayer
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from cpskin.core.utils import add_leadimage_from_file
from cpskin.menu.interfaces import IDirectAccess
from datetime import datetime
from DateTime import DateTime
from datetime import timedelta
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.schemaeditor.utils import FieldAddedEvent
from plone.schemaeditor.utils import IEditableSchema
from plone.uuid.interfaces import IUUID
from Products.statusmessages.interfaces import IStatusMessage
from zope import schema
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.event import notify
from zope.interface import alsoProvides
from zope.interface import directlyProvides
from zope.lifecycleevent import ObjectAddedEvent

import pytz
import unittest


class TestViews(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        fti = api.portal.get_tool('portal_types')['Plone Site']
        fti.allowed_content_types = fti.allowed_content_types + ('directory',)
        self.folder = api.content.create(self.portal, 'Folder', 'folder')

    def test_opendata_view(self):
        directlyProvides(self.request, ICPSkinCoreLayer)  # noqa
        configure_folderviews(self.portal)
        view = self.portal.restrictedTraverse('opendata')
        links = view.get_links()
        self.assertEqual(len(links), 3)

    def test_folderview_setting_named_link(self):
        add_behavior(
            'Collection',
            'cpskin.core.behaviors.indexview.ICpskinIndexViewSettings')
        configure_folderviews(self.portal)
        news = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=news, transition='publish')
        api.content.transition(obj=self.portal.actualites.actualites,
                               transition='publish')
        api.content.transition(obj=self.portal.actualites,
                               transition='publish')
        collection = self.portal.actualites.actualites
        link_text = getattr(collection, 'link_text')
        self.assertEqual(link_text, '')
        view = getMultiAdapter((self.portal, self.request), name='folderview')
        voir_lensemble_des = view.see_all(collection)
        self.assertEqual(voir_lensemble_des,
                         "Voir l'ensemble des actualit\xc3\xa9s")

        collection.link_text = u'Voir toutes les actualit\xc3\xa9s'
        voir_lensemble_des = view.see_all(collection)
        self.assertEqual(voir_lensemble_des,
                         'Voir toutes les actualit\xc3\x83\xc2\xa9s')
        self.assertTrue('Voir toutes les actualit' in view.index())

    def test_folderview_setting_image_scale(self):
        add_behavior(
            'Collection',
            'cpskin.core.behaviors.indexview.ICpskinIndexViewSettings')
        configure_folderviews(self.portal)
        news = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=news, transition='publish')
        collection = self.portal.actualites.actualites
        view = getMultiAdapter(
            (self.portal, self.request), name='folderview')
        self.assertEqual(collection.collection_image_scale, 'collection')
        self.assertEqual(collection.slider_image_scale, 'slider')
        self.assertEqual(collection.carousel_image_scale, 'carousel')
        scale = view.collection_image_scale(collection, news)
        self.assertFalse(scale)
        add_leadimage_from_file(news, 'visuel.png')
        scale = view.collection_image_scale(collection, news)
        self.assertTrue('height="116"' in scale)

        collection.collection_image_scale = 'thumb'
        scale = view.collection_image_scale(collection, news)
        self.assertTrue('height="250"' in scale)

    def test_folderview_add_remove_content(self):
        configure_folderviews(self.portal)
        request = self.portal.actualites.REQUEST
        api.content.create(
            container=self.folder,
            type='News Item',
            id='testnewsitem')
        view = getMultiAdapter(
            (self.portal.actualites, request), name='folderview')
        self.assertTrue(view.canRemoveContent())
        self.assertFalse(view.canAddContent())
        view.removeContent()
        self.assertFalse(view.canRemoveContent())
        self.assertTrue(view.canAddContent())
        view.addContent()
        self.assertTrue(view.canRemoveContent())
        self.assertFalse(view.canAddContent())

    def test_folderview_render(self):
        configure_folderviews(self.portal)
        request = self.portal.REQUEST
        testnewsitem = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=testnewsitem, transition='publish')
        api.content.transition(obj=self.portal.actualites,
                               transition='publish')
        view = getMultiAdapter((self.portal, request), name='folderview')
        self.assertIn(
            u'<a href="http://nohost/plone/actualites/actualites" title="">Actualit\xe9s',
            view.index())

    def test_folderview_event_category(self):
        applyProfile(self.portal, 'collective.taxonomy:default')
        add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)

        utility = queryUtility(ITaxonomy, name='collective.taxonomy.test')
        collection = api.content.create(container=self.folder,
                                        type='Collection',
                                        id='collection')
        collection.taxonomy_category = 'taxonomy_test'
        collection.reindexObject()

        taxonomy_test = schema.Set(
            title=u'taxonomy_test',
            description=u'taxonomy description schema',
            required=False,
            value_type=schema.Choice(
                vocabulary=u'collective.taxonomy.test'),
        )
        portal_types = api.portal.get_tool('portal_types')
        fti = portal_types.get('Event')
        event_schema = fti.lookupSchema()
        schemaeditor = IEditableSchema(event_schema)
        schemaeditor.addField(taxonomy_test, name='taxonomy_test')
        notify(ObjectAddedEvent(taxonomy_test, event_schema))
        notify(FieldAddedEvent(fti, taxonomy_test))
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='testevent')
        simple_tax = [val for val in utility.data['en'].values()]
        event.taxonomy_test = set(simple_tax[0])

        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='folderview')

        see_categories = view.see_categories(collection)
        self.assertTrue(see_categories)
        collection.taxonomy_category = ''
        see_categories = view.see_categories(collection)
        self.assertFalse(see_categories)
        collection.taxonomy_category = 'taxonomy_test'
        categories = view.get_categories(collection, event)
        self.assertEqual(categories, 'Information Science')

        event.taxonomy_test = set(simple_tax[0:2])
        categories = view.get_categories(collection, event)
        self.assertEqual(
            categories,
            'Book Collecting, Information Science')

        event.taxonomy_test = set()
        categories = view.get_categories(collection, event)
        self.assertEqual(categories, '')

    def test_folderview_event_item_count_homepage(self):
        add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)

        collection = api.content.create(container=self.folder,
                                        type='Collection',
                                        id='collection')

        collection.setQuery(
            [{u'i': u'portal_type',
              u'o': u'plone.app.querystring.operation.selection.is',
              u'v': [u'News Item']}, ]
        )
        collection.item_count_homepage = 1
        api.content.create(
            container=self.folder,
            type='News Item',
            id='testnews')
        api.content.create(
            container=self.folder,
            type='News Item',
            id='testnews2')

        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='folderview')

        result = view.getResults(collection)
        self.assertEqual(len(result['standard-results']), 1)

        collection.item_count_homepage = 2
        result = view.getResults(collection)
        self.assertEqual(len(result['standard-results']), 2)

    def test_folderview_event_localizedtime(self):
        utc = pytz.utc
        start = datetime(2001, 1, 1, 10, 0, tzinfo=utc)
        end = datetime(2001, 1, 1, 11, 0, tzinfo=utc)
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='testevent',
            start=start,
            end=end,
            timezone='UTC')

        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='folderview')

        oneday = view.is_one_day(event)
        self.assertTrue(oneday)

        event.end = datetime(2001, 1, 2, 11, 0, tzinfo=utc)
        oneday = view.is_one_day(event)
        self.assertFalse(oneday)

        withhours = view.is_with_hours(event)
        self.assertTrue(withhours)

    def test_event_geo_contents_view(self):
        add_behavior('Event', ICoordinates.__identifier__)
        event = api.content.create(container=self.folder,
                                   type='Event', title='document')
        event.location = 'Rue Léon Morel, 1 5032 Isnes'
        form = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='set-geo-contents-form')
        form.request.form = {'form.widgets.content_types': [u'Event']}
        form.update()
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        coord = ICoordinates(event).coordinates
        self.assertFalse(coord.startswith('POINT '))
        form.handleApply(form, 'Ok')
        coord = ICoordinates(event).coordinates
        self.assertTrue(coord.startswith('POINT '))

    def test_orga_geo_contents_view(self):
        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')
        person.street = u'Rue de la Vieille Sambre'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'

        add_behavior('person', ICoordinates.__identifier__)

        form = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='set-geo-contents-form')
        form.request.form = {'form.widgets.content_types': [u'person']}
        form.update()

        coord = ICoordinates(person).coordinates
        self.assertFalse(coord.startswith('POINT '))
        form.handleApply(form, 'Ok')
        coord = ICoordinates(person).coordinates
        self.assertTrue(coord.startswith('POINT '))

    def test_bad_orga_geo_contents_view(self):
        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        add_behavior('person', ICoordinates.__identifier__)
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')

        form = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='set-geo-contents-form')
        form.request.form = {'form.widgets.content_types': [u'person']}
        form.update()
        form.handleApply(form, 'Ok')
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()  # noqa
        self.assertEqual(
            messages[0].message,
            u'No address for http://nohost/plone/directory/person')  # noqa
        self.assertEqual(messages[1].message, u'0 person are updated')
        person.street = u'Rue de l\'hôtel de ville'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'

        form = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='set-geo-contents-form')
        form.request.form = {'form.widgets.content_types': [u'person']}
        form.update()

        coord = ICoordinates(person).coordinates
        self.assertFalse(coord.startswith('POINT '))
        form.handleApply(form, 'Ok')
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()  # noqa
        self.assertEqual(messages[0].message, u'1 person are updated')
        coord = ICoordinates(person).coordinates
        self.assertTrue(coord.startswith('POINT '))

    def test_date_event_generation_helper_view(self):
        applyProfile(self.portal, 'collective.documentgenerator:default')
        utc = pytz.utc
        start = datetime(2001, 1, 1, 10, 0, tzinfo=utc)
        end = datetime(2001, 1, 1, 11, 0, tzinfo=utc)
        event = api.content.create(container=self.folder,
                                   type='Event', title='my_event',
                                   start=start, end=end, timezone='UTC')
        view = getMultiAdapter(
            (event, self.portal.REQUEST),
            name='document_generation_helper_view')
        view.real_context = event
        self.assertEqual(
            view.get_formatted_date(), u'1 January 2001 de 10:00 \xe0 11:00')

        event.end = datetime(2001, 1, 3, 11, 0, tzinfo=utc)
        view = getMultiAdapter(
            (event, self.portal.REQUEST),
            name='document_generation_helper_view')
        self.assertEqual(
            view.get_formatted_date(), u'1 au 3 January 2001 de 10:00 \xe0 11:00')

        event.end = datetime(2001, 2, 1, 11, 0, tzinfo=utc)
        view = getMultiAdapter(
            (event, self.portal.REQUEST),
            name='document_generation_helper_view')
        self.assertEqual(
            view.get_formatted_date(),
            u'1 January 2001 au 1 February 2001 de 10:00 \xe0 11:00')

        event.end = datetime(2001, 1, 1, 11, 0, tzinfo=utc)
        event.open_end = True
        view = getMultiAdapter(
            (event, self.portal.REQUEST),
            name='document_generation_helper_view')
        self.assertEqual(
            view.get_formatted_date(), u'1 January 2001 \xe0 10:00')

        event.contact = 'Imio'
        event.phone = '081/586.100'
        info = view.get_info()
        self.assertEqual(info, 'Info : my_event - 081 58 61 00')

    def test_folderview_hide_title(self):
        # directlyProvides(self.portal.REQUEST, ICPSkinCoreLayer)  # noqa
        add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)
        configure_folderviews(self.portal)
        collection = self.portal.actualites.actualites
        collection.title = u'My test collection'
        collection.setQuery(
            [{u'i': u'portal_type',
              u'o': u'plone.app.querystring.operation.selection.is',
              u'v': [u'News Item']}, ]
        )
        collection.reindexObject()
        news = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnews')
        news.title = u'My test news'

        api.content.transition(obj=collection, transition='publish')
        api.content.transition(obj=self.portal.actualites,
                               transition='publish')
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        self.assertFalse(view.hide_title(collection))
        self.assertIn('<h2>My test collection</h2>', view())

        collection.hide_title = True
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        self.assertTrue(view.hide_title(collection))
        self.assertNotIn('<h2>My test collection</h2>', view())

    def test_folderview_hide_see_all_link(self):
        add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)
        configure_folderviews(self.portal)
        collection = self.portal.actualites.actualites
        collection.setQuery(
            [{u'i': u'portal_type',
              u'o': u'plone.app.querystring.operation.selection.is',
              u'v': [u'News Item']}, ]
        )
        api.content.create(
            container=self.folder,
            type='News Item',
            id='testnews')
        api.content.transition(obj=collection, transition='publish')
        api.content.transition(obj=self.portal.actualites,
                               transition='publish')
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        self.assertFalse(view.hide_see_all_link(collection))
        self.assertIn(u"Voir l\'ensemble des actualit\xe9s", view())

        collection.hide_see_all_link = True
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        self.assertTrue(view.hide_see_all_link(collection))
        self.assertNotIn(u"Voir l\'ensemble des actualit\xe9s", view())

    def test_folderview_hide_date(self):
        add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)
        configure_folderviews(self.portal)
        collection = self.portal.actualites.actualites
        collection.setQuery(
            [{u'i': u'portal_type',
              u'o': u'plone.app.querystring.operation.selection.is',
              u'v': [u'News Item']}, ]
        )
        news = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnews')
        api.content.transition(obj=news, transition='publish')
        news.setEffectiveDate(DateTime('2016/07/26'))
        news.reindexObject()
        api.content.transition(obj=collection, transition='publish')
        api.content.transition(obj=self.portal.actualites,
                               transition='publish')
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        news_brain = view.getResults(collection)['standard-results'][0]
        self.assertFalse(view.hide_date(news_brain, collection))
        self.assertIn(u'Jul 26, 2016', view())

        collection.hide_date = True
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='folderview')
        self.assertTrue(view.hide_date(news_brain, collection))
        self.assertNotIn(u'Jul 26, 2016', view())

    def test_teleservice_template(self):
        view = api.content.get_view(
            name='teleservice-template',
            context=self.portal,
            request=self.request)
        self.assertIn('id="teleservice-nav"', view())

    def test_folderview_old_new_template(self):
        add_behavior(
            'Collection',
            'cpskin.core.behaviors.indexview.ICpskinIndexViewSettings')
        configure_folderviews(self.portal)
        news = api.content.create(
            container=self.folder,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=news, transition='publish')
        collection = self.portal.actualites.actualites
        view = getMultiAdapter(
            (self.portal, self.request), name='folderview')
        self.assertEqual(collection.collection_image_scale, 'collection')
        add_leadimage_from_file(news, 'visuel.png')
        scale = view.collection_image_scale(collection, news)
        self.assertTrue('height="116"' in scale)

        collection.use_new_template = True
        scale = view.collection_image_scale(collection, news)
        self.assertEqual(scale.height, 116)
        self.assertEqual(scale.width, 77)

    def test_cpskin_navigation_view(self):
        applyProfile(self.portal, 'cpskin.workflow:default')
        self.portal.portal_workflow.setDefaultChain('cpskin_workflow')
        subfolder = api.content.create(self.folder, 'Folder', 'subfolder')
        subsubfolder = api.content.create(subfolder, 'Folder', 'subfolder')
        alsoProvides(subsubfolder, IDirectAccess)
        subsubfolder.reindexObject()
        view = api.content.get_view(
            name='cpskin_navigation_view',
            context=self.folder,
            request=self.folder.REQUEST)
        self.assertEqual(0, len(view.menus()))
        api.content.transition(obj=subfolder, transition='publish_and_show')
        self.assertEqual(1, len(view.menus()))
        self.assertEqual(0, len(view.accesses()))
        view = api.content.get_view(
            name='cpskin_navigation_view',
            context=subfolder,
            request=subfolder.REQUEST)
        self.assertEqual(1, len(view.accesses()))

    def test_cpskinhealthy_view(self):
        applyProfile(self.portal, 'cpskin.workflow:default')
        view = self.portal.restrictedTraverse('cpskinhealthy')
        contacts = view.contacts()
        self.assertEqual(contacts['is_installed'], False)
        view.install_contact_core()
        contacts = view.contacts()
        self.assertEqual(contacts['is_installed'], True)
        self.assertEqual(contacts['is_cpskin_workflow'], False)
        view.set_contact_worflow()
        contacts = view.contacts()
        self.assertEqual(contacts['is_cpskin_workflow'], True)

        self.assertEqual(view.get_site_language(), 'en')
        self.assertFalse(view.is_site_language_fr())
        view.set_site_language()
        self.assertEqual(view.get_site_language(), 'fr')
        self.assertTrue(view.is_site_language_fr())

    def test_cpskin_navigation_with_leadimage_view(self):
        applyProfile(self.portal, 'cpskin.workflow:default')
        add_behavior(
            'Folder',
            'plone.app.contenttypes.behaviors.leadimage.ILeadImage')
        self.portal.portal_workflow.setDefaultChain('cpskin_workflow')
        subfolder = api.content.create(self.folder, 'Folder', 'subfolder')
        view = api.content.get_view(
            name='cpskin_navigation_view_with_leadimage',
            context=self.folder,
            request=self.folder.REQUEST)
        self.assertEqual(0, len(view.menus()))
        api.content.transition(obj=subfolder, transition='publish_and_show')
        self.assertEqual(1, len(view.menus()))
        brain = api.content.find(UID=IUUID(subfolder))[0]
        self.assertFalse(view.image(brain))
        add_leadimage_from_file(subfolder, 'visuel.png')
        self.assertTrue(view.image(brain).startswith(
            u'http://nohost/plone/folder/subfolder/@@images'
        ))

    def test_cpskin_search_view(self):
        api.content.create(
            container=self.portal,
            type='Document',
            id='document',
            title='My testing document'
        )
        request = self.portal.REQUEST
        request.form = {'SearchableText': 'testi'}
        view = api.content.get_view(
            name='search',
            context=self.portal,
            request=request)
        query = view.filter_query({})
        self.assertEqual(query['SearchableText'], 'testi*')
        self.assertEqual(len(view.results()), 1)

    def test_replace_rich_text_view(self):
        doc = api.content.create(self.portal, 'Document', 'doc')
        text = '<a href="http://nohost/plone/at_download/file">link</a>'
        doc.text = RichTextValue(text)
        form = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='replace-richtext-form')
        form.request.form = {
            'form.widgets.old_text': '/at_download/file',
            'form.widgets.new_text': ''
        }
        form.update()
        self.assertTrue('at_download/file' in doc.text.raw)
        form.handleApply(form, 'Ok')
        self.assertFalse('at_download/file' in doc.text.raw)



# -*- coding: utf-8 -*-
from cpskin.core.utils import has_lat_lng
from cpskin.core.utils import set_coord
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from plone.app.textfield.value import RichTextValue
from plone.directives import form
from z3c.form import button
from zope import schema

import logging


logger = logging.getLogger('cpskin.core.encode_lat_lng')


class IGeoForm(form.Schema):
    """ Define form fields """

    content_types = schema.List(
        title=_(u'Content type'),
        description=_(
            u'Which content type should be set latitude and longitude'),
        value_type=schema.Choice(
            title=_(u'Content types'),
            vocabulary='cpskin.core.vocabularies.geo_types',
        ),
        required=False,
    )


class GeoForm(form.SchemaForm):

    schema = IGeoForm
    ignoreContext = True

    label = u"What's object do you want to update ?"
    description = u'This script will update latitude and longitude for objects \
                    selected'

    @button.buttonAndHandler(u'Ok')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        # Do something with valid data here

        # Set status on this form page (this status message
        # is not bind to the session and does not go thru redirects)
        results = []
        if 'content_types' not in data.keys():
            pass
        else:
            for portal_type in data['content_types']:
                list_updated = set_lat_lng(portal_type, self.request)
                results.extend(list_updated)
                message = '{1} {0} are updated'.format(
                    portal_type, len(list_updated))
                api.portal.show_message(message=message, request=self.request)
        # self.status = "\n".join(results)

    @button.buttonAndHandler(u"Cancel")
    def handleCancel(self, action):
        """User cancelled. Redirect back to the front page.
        """


def set_lat_lng(portal_type, request):
    catalog = api.portal.get_tool('portal_catalog')
    query = {}
    query['portal_type'] = portal_type
    brains = catalog(query)
    nbre = len(brains)
    results = []
    i = 0
    for brain in brains:
        obj = brain.getObject()
        i += 1
        if not has_lat_lng(obj):
            message = set_coord(obj, request)
            if message:
                results.append(message)
        logger.info('{0}/{1}'.format(i, nbre))
    return results


class IReplaceRichtextForm(form.Schema):
    """ Define form fields """

    old_text = schema.TextLine(
        title=_(u'Old text'),
        description=_(
            u'Text to be replaced in all your RichTextValue'),
        required=False,
    )

    new_text = schema.TextLine(
        title=_(u'New text'),
        description=_(
            u'Text which will replace all your old_text into RichTextValue'),
        required=False,
    )


class ReplaceRichtextForm(form.SchemaForm):
    schema = IReplaceRichtextForm
    ignoreContext = True

    label = u'What text do you want to change ?'
    description = u'This script will update old_text value with new_text value'

    @button.buttonAndHandler(u'Ok')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        # Do something with valid data here

        # Set status on this form page (this status message
        # is not bind to the session and does not go thru redirects)
        old_text = data['old_text']
        new_text = data['new_text']
        if not old_text:
            old_text = u''
        if not new_text:
            new_text = u''
        catalog = api.portal.get_tool('portal_catalog')
        query = {}
        query['object_provides'] = 'plone.app.contenttypes.behaviors.richtext.IRichText'  # noqa
        results = catalog(**query)
        for result in results:
            obj = result.getObject()
            text = getattr(obj, 'text', None)
            if text:
                clean_text = text.raw.replace(old_text, new_text)
                if clean_text != text.raw:
                    obj.text = RichTextValue(
                        raw=clean_text,
                        mimeType=text.mimeType,
                        outputMimeType=text.outputMimeType,
                        encoding=text.encoding
                    )
                    obj.reindexObject()
                    message = '{0} is updated'.format(obj.absolute_url())
                    logger.info(message)
                    api.portal.show_message(
                        message=message, request=self.request)



# -*- coding: utf-8 -*-
from cpskin.core.menu.interfaces import IConfigurationsMenu
from cpskin.core.menu.interfaces import IConfigurationsSubMenuItem
from cpskin.locales import CPSkinMessageFactory as _
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from zope.browsermenu.menu import BrowserMenu
from zope.browsermenu.menu import BrowserSubMenuItem
from zope.component import getMultiAdapter
from zope.interface import implements


class ConfigurationsSubMenuItem(BrowserSubMenuItem):
    implements(IConfigurationsSubMenuItem)

    title = _(u'label_cpskin_configurations_menu', default=u'Configurations')
    description = _(
        u'title_cpskin_actions_menu',
        default=u'Configurations for the current content item')
    submenuId = 'plone_contentmenu_cpskin_configurations'

    order = 15
    extra = {'id': 'plone-contentmenu-cpskin-configurations'}

    @property
    def action(self):
        return self.context.absolute_url()

    @memoize
    def available(self):
        actions_tool = getToolByName(self.context, 'portal_actions')
        actions = actions_tool.listActionInfos(
            object=self.context,
            categories=('cpskin_configurations',), max=1)
        return len(actions) > 0

    def selected(self):
        return False


class ConfigurationsMenu(BrowserMenu):
    implements(IConfigurationsMenu)

    def getMenuItems(self, context, request):
        """Return menu item entries in a TAL-friendly form."""
        results = []

        context_state = getMultiAdapter(
            (context, request),
            name='plone_context_state')
        actions = context_state.actions('cpskin_configurations')
        if not actions:
            return results
        for action in actions:
            if action['allowed']:
                aid = action['id']

                results.append({
                    'title': action['title'],
                    'description': '',
                    'action': action['url'],
                    'selected': False,
                    'icon': None,
                    'extra': {'id': 'plone-contentmenu-cpskin-configurations-' + aid,  # noqa
                              'separator': None,
                              'class': None},
                    'submenu': None,
                })
        return results



# -*- coding: utf-8 -*-
from Acquisition import aq_get
from binascii import b2a_qp
from collective.geo.behaviour.interfaces import ICoordinates
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from plone.supermodel.interfaces import FIELDSETS_KEY
from plone.supermodel.utils import mergedTaggedValueList
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from zope.component import getUtility
from zope.component.interface import nameToInterface
from zope.i18n import translate
from zope.interface import implements
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

from cpskin.locales import CPSkinMessageFactory as _


class BookingTypesFactory(object):
    """Vocabulary factory listing booking types"""
    implements(IVocabularyFactory)
    indexName = ''

    def __call__(self, context, query=None):
        items = [
            ('no_booking', _(u'No booking')),
            ('mandatory', _(u'Mandatory booking')),
            ('optional', _(u'Optional booking')),
        ]
        terms = [
            SimpleTerm(
                value=pair[0],
                token=pair[0],
                title=pair[1],
            ) for pair in items
        ]
        return SimpleVocabulary(terms)


BookingTypesVocabularyFactory = BookingTypesFactory()


class BaseTagsVocabulary(object):
    """Vocabulary factory listing all catalog keywords from specified tags"""
    implements(IVocabularyFactory)
    indexName = ''

    def __call__(self, context, query=None):
        site = api.portal.get()
        self.catalog = getToolByName(site, 'portal_catalog', None)
        if self.catalog is None:
            return SimpleVocabulary([])
        if self.indexName not in self.catalog._catalog.indexes.keys():
            return SimpleVocabulary([])
        index = self.catalog._catalog.getIndex(self.indexName)

        def safe_encode(term):
            if isinstance(term, unicode):
                # no need to use portal encoding for transitional encoding from
                # unicode to ascii. utf-8 should be fine.
                term = term.encode('utf-8')
            return term

        # Vocabulary term tokens *must* be 7 bit values, titles *must* be
        # unicode
        items = [
            SimpleTerm(safe_unicode(i), b2a_qp(
                safe_encode(i)), safe_unicode(i))
            for i in index._index
            if query is None or safe_encode(query) in safe_encode(i)
        ]
        return SimpleVocabulary(items)


class IStandardTagsVocabulary(BaseTagsVocabulary):
    """Vocabulary factory listing all catalog keywords from standard tags"""
    indexName = 'standardTags'


class IAmTagsVocabulary(BaseTagsVocabulary):
    """Vocabulary factory listing all catalog keywords from I am tags"""
    indexName = 'iamTags'


class ISearchTagsVocabulary(BaseTagsVocabulary):
    """Vocabulary factory listing all catalog keywords from I search tags"""
    indexName = 'isearchTags'


class HiddenTagsVocabulary(BaseTagsVocabulary):
    """Vocabulary factory listing all catalog keywords from hidden tags"""
    indexName = 'hiddenTags'

    def __call__(self, context, query=None):
        voc = super(HiddenTagsVocabulary, self).__call__(context, query)
        # Add a-la-une term if it does not already exist, aka still not used
        # for any document
        if u'a-la-une' not in voc.by_token and u'A la une' not in voc.by_token:
            simpleTerms = [term for term in voc]
            simpleTerms.append(SimpleTerm(u'a-la-une', title=u'A la une'))
            voc = SimpleVocabulary(simpleTerms)
        if u'homepage' not in voc.by_token:
            simpleTerms = [term for term in voc]
            simpleTerms.append(SimpleTerm(u'homepage', title=u'homepage'))
            voc = SimpleVocabulary(simpleTerms)
        return voc


IStandardTagsVocabularyFactory = IStandardTagsVocabulary()
IAmTagsVocabularyFactory = IAmTagsVocabulary()
ISearchTagsVocabularyFactory = ISearchTagsVocabulary()
HiddenTagsVocabularyFactory = HiddenTagsVocabulary()


class ContactFieldsFactory(object):
    """Vocabulary factory listing all fields from contacts"""
    implements(IVocabularyFactory)
    indexName = ''

    def __call__(self, context, query=None):
        results = []
        field_ids = []
        exclude = ['im_handle', 'use_parent_address']
        exclude_behaviors = ['plone.app.content.interfaces.INameFromTitle']
        behaviors = set()
        portal_types = api.portal.get_tool('portal_types')
        contact_portal_types = ['person', 'organization', 'position', 'held_position']  # noqa
        for contact_portal_type in contact_portal_types:
            schema = getUtility(
                IDexterityFTI, name=contact_portal_type).lookupSchema()
            fieldsets = mergedTaggedValueList(schema, FIELDSETS_KEY)
            for name, field in getFieldsInOrder(schema):
                if name not in exclude and name not in field_ids:
                    visible_name = u'{0}: {1}'.format(
                        contact_portal_type, field.title)
                    field_ids.append(name)
                    results.append((name, visible_name))

            portal_type = getattr(portal_types, contact_portal_type)
            behaviors.update(set(portal_type.behaviors))
        try:
            # remove duplicates photo
            results.remove(('photo', u'held_position: Photo'))
        except ValueError:
            pass
        for behavior in behaviors:
            if behavior not in exclude_behaviors:
                try:
                    # not able to get fields from IDirectoryContactDetails
                    # with nameToInterface(context, behavior)
                    if behavior == 'cpskin.core.behaviors.directorycontact.IDirectoryContactDetails':  # noqa
                        from cpskin.core.behaviors.directorycontact import (
                            IDirectoryContactDetails)
                        interface = IDirectoryContactDetails
                    else:
                        interface = nameToInterface(context, behavior)
                    fieldsets = mergedTaggedValueList(interface, FIELDSETS_KEY)
                    for name, field in getFieldsInOrder(interface):
                        if name not in exclude and name not in field_ids:
                            if not fieldsets:
                                visible_name = field.title
                            else:
                                fieldset = [
                                    fieldset for fieldset in fieldsets if name in fieldset.fields  # noqa
                                ][0]
                                field_ids.append(name)
                                visible_name = u'{0}: {1}'.format(
                                    fieldset.label, field.title)
                            results.append((name, visible_name))
                except:
                    pass
        items = [
            SimpleTerm(i, i, j)
            for i, j in results if j
        ]
        return SimpleVocabulary(items)


ContactFieldsVocabularyFactory = ContactFieldsFactory()


class GeoTypesFactory(object):
    implements(IVocabularyFactory)

    def __call__(self, context, query=None):
        portal_types = api.portal.get_tool('portal_types')
        request = aq_get(portal_types, 'REQUEST', None)
        if portal_types is None:
            return SimpleVocabulary([])

        items = []

        for k, v in portal_types.items():
            behaviors = getattr(v, 'behaviors', [])
            if ICoordinates.__identifier__ in behaviors:
                items.append(
                        (k, translate(v.Title(), context=request))
                )
        items.sort(key=lambda x: x[1])
        items = [SimpleTerm(i[0], i[0], i[1]) for i in items]
        return SimpleVocabulary(items)


GeoTypesVocabularyFactory = GeoTypesFactory()


class ActionMenuEligibleFactory(object):
    implements(IVocabularyFactory)

    def __call__(self, context, query=None):
        root = api.portal.get()
        if 'fr' in root.objectIds():
            fr = getattr(root, 'fr')
            root = api.portal.get_navigation_root(fr)

        rootPath = '/'.join(root.getPhysicalPath())
        query = {}
        query['path'] = {'query': rootPath, 'depth': 1}
        query['portal_type'] = ['Folder']
        query['sort_on'] = 'sortable_title'
        query['is_default_page'] = False
        portal_catalog = api.portal.get_tool('portal_catalog')
        results = portal_catalog.searchResults(query)
        items = []
        for result in results:
            if not result.exclude_from_nav:
                continue
            items.append(
                SimpleTerm(
                    safe_unicode(result.id),
                    safe_unicode(result.id),
                    safe_unicode(result.Title),
                )
            )
        return SimpleVocabulary(items)


ActionMenuEligibleVocabularyFactory = ActionMenuEligibleFactory()


DISPLAY_TYPES = {
    u'slider-with-carousel': {
        'value': u'slider-with-carousel',
        'title': _(u'Slider with carousel'),
        'slider': True,
        'class': 'slider-carousel',
        'control-nav': False,
        'show-image': True,
        'show-carousel': True,
    },
    u'slider-with-elements-count-choice': {
        'value': u'slider-with-elements-count-choice',
        'title': _(u'Slider with elements count choice'),
        'slider': True,
        'class': 'slider-multiple',
        'control-nav': True,
        'show-image': True,
        'show-carousel': True,
    },
    u'slider-slick': {
        'value': u'slider-slick',
        'title': _(u'Slider Slick'),
        'slider': True,
        'class': 'slider-slick-mode',
        'control-nav': True,
        'show-image': True,
        'show-carousel': True,
        'slick': True,
    },
    u'slider-slick-full-width': {
        'value': u'slider-slick-full-width',
        'title': _(u'Slider Slick Full Width'),
        'slider': True,
        'class': 'slider-slick-mode',
        'control-nav': True,
        'show-image': True,
        'show-carousel': True,
        'variable-width': True,
        'slick': True,
    },
    u'unique-slider-with-title-carousel': {
        'value': u'unique-slider-with-title-carousel',
        'title': _(u'Unique slider with carousel on title'),
        'slider': True,
        'class': 'slider-unique-titre',
        'control-nav': True,
        'show-image': True,
        'show-carousel': True,
    },
    u'highlighted-unique-item': {
        'value': u'highlighted-unique-item',
        'title': _(u'Unique highlighted item'),
        'slider': False,
        'class': 'element-en-evidence',
        'control-nav': False,
        'show-image': True,
        'show-carousel': True,
    },
    u'slider-without-images': {
        'value': u'slider-without-images',
        'title': _(u'Slider without images'),
        'slider': True,
        'class': 'slider-without-images',
        'control-nav': True,
        'show-image': False,
        'show-carousel': False,
    },
    u'slider-vertical': {
        'value': u'slider-vertical',
        'title': _(u'Slider vertical'),
        'slider': True,
        'class': 'slider-vertical',
        'control-nav': None,
        'show-image': None,
        'show-carousel': None,
    },
    u'slider-count': {
        'value': u'slider-count',
        'title': _(u'Slider count'),
        'slider': True,
        'class': 'slider-count',
        'control-nav': True,
        'show-image': True,
        'show-carousel': False,
    },
    u'slider-top': {
        'value': u'slider-top',
        'title': _(u'Slider top'),
        'slider': True,
        'class': 'slider-top',
        'control-nav': True,
        'show-image': True,
        'show-carousel': False,
    },
}

items = [(disp['value'], disp['title']) for disp in DISPLAY_TYPES.values()]
items.sort(key=lambda x: x[1])
index_view_display_type = SimpleVocabulary(
    [SimpleTerm(i[0], i[0], i[1]) for i in items]
)


class SliderDisplayTypeVocabularyFactory(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        use_slick = api.portal.get_registry_record(
            "cpskin.core.interfaces.ICPSkinSettings.use_slick",
            default=False,
        )
        items = [(disp['value'], disp['title']) for disp in DISPLAY_TYPES.values()]
        if use_slick is True:
            items = [e for e in items if DISPLAY_TYPES[e[0]].get("slick", False)]
        else:
            items = [e for e in items if not DISPLAY_TYPES[e[0]].get("slick", False)]
        items.sort(key=lambda x: x[1])
        return SimpleVocabulary(
            [SimpleTerm(i[0], i[0], i[1]) for i in items]
        )


SliderDisplayTypeVocabulary = SliderDisplayTypeVocabularyFactory()



# -*- coding: utf-8 -*-

from Products.Five.browser import BrowserView
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager

import os
import unittest


def dummy_content(filename):
    file_path = os.path.join(os.path.dirname(__file__), filename)
    return NamedBlobImage(
        data=open(file_path, 'r').read(),
        filename=file_path
    )


class TestBanners(unittest.TestCase):
    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        manager_name = 'plone.portaltop'
        view = BrowserView(self.portal, self.request)
        manager = queryMultiAdapter(
            (self.portal, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        manager.update()
        self.my_viewlet = [v for v in manager.viewlets
                           if v.__name__ == 'cpskin.banner'][0]

    def test_banner_without_folder(self):
        banner = self.my_viewlet.getBanner()
        url = banner.get('url')
        self.assertTrue(url.startswith('http://nohost/plone/banner.jpg'))
        self.assertEqual(banner.get('type'), 'image')

    def test_banner_with_folder_without_image(self):
        api.content.create(
            container=self.portal, type='Folder', id='banner')
        banner = self.my_viewlet.getBanner()
        url = banner.get('url')
        self.assertTrue(url.startswith('http://nohost/plone/banner.jpg'))
        self.assertEqual(banner.get('type'), 'image')

    def test_banner_with_folder_with_image(self):
        folder = api.content.create(
            container=self.portal, type='Folder', id='banner')
        image = api.content.create(
            container=folder, type='Image', id='img1.png')
        image.image = dummy_content(u'banner.png')
        banner = self.my_viewlet.getBanner()
        url = banner.get('url')
        self.assertTrue(url.startswith('http://nohost/plone/banner/img1.png'))
        self.assertEqual(banner.get('type'), 'image')
    
    def test_banner_with_folder_with_one_video(self):
        folder = api.content.create(
            container=self.portal, type='Folder', id='banner')
        image = api.content.create(
            container=folder, type='Image', id='img1.png')
        image.image = dummy_content(u'banner.png')
        video = api.content.create(
            container=folder, type='File', id='banner.mp4')
        video.file = dummy_content(u'banner.mp4')
        banner = self.my_viewlet.getBanner()
        url = banner.get('url')
        self.assertTrue(url.startswith('http://nohost/plone/banner/img1.png'))
        self.assertEqual(banner.get('type'), 'image')


    def test_banner_with_folder_with_two_video(self):
        folder = api.content.create(
            container=self.portal, type='Folder', id='banner')
        image = api.content.create(
            container=folder, type='Image', id='img1.png')
        image.image = dummy_content(u'banner.png')
        video = api.content.create(
            container=folder, type='File', id='banner.mp4')
        video.file = dummy_content(u'banner.mp4')
        video = api.content.create(
            container=folder, type='File', id='banner.webm')
        video.file = dummy_content(u'banner.webm')
        banner = self.my_viewlet.getBanner()
        url = banner.get('url')
        url_webm = banner.get('url_webm')
        self.assertTrue(url.startswith('http://nohost/plone/banner/banner.mp4'))
        self.assertTrue(url_webm.startswith('http://nohost/plone/banner/banner.webm'))
        self.assertEqual(banner.get('type'), 'video')



# -*- coding: utf-8 -*-
from cpskin.core.vocabulary import index_view_display_type
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.directives import form
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class ICpskinIndexViewSettings(model.Schema):
    model.fieldset(
        'indexview',
        label=_(
            u'Index view'),
        fields=(
            'display_type',
            'minimum_items_in_slider',
            'maximum_items_in_slider',
            'slide_to_scroll',
            'speed',
            'show_arrows',
            'show_dots',
            'breakpoint_full',
            'slides_to_show_full',
            'slides_to_scroll_full',
            'breakpoint_medium',
            'slides_to_show_medium',
            'slides_to_scroll_medium',
            'breakpoint_small',
            'slides_to_show_small',
            'slides_to_scroll_small',
            'use_center_mode',
            'center_padding',
            'autoplay_mode',
            'autoplay_speed',
            'fade',
            'collection_image_scale',
            'slider_image_scale',
            'carousel_image_scale',
            'link_text',
            'index_view_keywords',
            'taxonomy_category',
            'item_count_homepage',
            'hide_title',
            'hide_see_all_link',
            'hide_date',
            'show_day_and_month',
            'show_lead_image',
            'show_descriptions',
            'use_new_template',
            'show_event_category_below_image',
        ),
    )

    display_type = schema.Choice(
        title=_(u'Collection display type'),
        description=_(u'How do you want to display this collection on index view ?'),  # noqa
        required=False,
        vocabulary=u"cpskin.core.vocabularies.slider_display_types",
    )

    minimum_items_in_slider = schema.Int(
        title=_(u'Minimum number of items in slider'),
        description=_(u'How many items do you want to see at least in slider (used only for "Slider with elements count choice" display type) ?'),  # noqa
        required=True,
        default=2,
        min=1,
        max=5,
    )

    maximum_items_in_slider = schema.Int(
        title=_(u'Maximum number of items in slider'),
        description=_(u'How many items do you want to see at most in slider (used only for "Slider with elements count choice" display type) ?'),  # noqa
        required=True,
        default=4,
        min=1,
        max=7,
    )

    collection_image_scale = schema.Choice(
        title=_(u'Which image scale use for collections of index view'),
        description=_(u'Please select which fields should be visible.'),
        required=False,
        default='collection',
        vocabulary=u'plone.app.vocabularies.ImagesScales'
    )

    slider_image_scale = schema.Choice(
        title=_(u'Which image scale use for slider'),
        description=_(u''),
        required=True,
        default='slider',
        vocabulary=u'plone.app.vocabularies.ImagesScales'
    )

    carousel_image_scale = schema.Choice(
        title=_(u'Which image scale use for carousel'),
        description=_(u'Please select which fields should be visible.'),
        required=True,
        default='carousel',
        vocabulary=u'plone.app.vocabularies.ImagesScales'
    )

    link_text = schema.TextLine(
        title=_(u'Text for link to collection'),
        description=_(
            u'This text will be visible on index view for link to this collection'),  # noqa
        default=_(u''),
        required=False
    )

    form.widget('index_view_keywords', CheckBoxFieldWidget)
    index_view_keywords = schema.List(
        title=_(u'Hidden keywords'),
        description=_(
            u'Please select which hidden keywords is use by collections for \
            index view.'),
        required=False,
        value_type=schema.Choice(
            vocabulary=u'cpskin.core.vocabularies.hiddenTags'
        )
    )

    taxonomy_category = schema.TextLine(
        title=_(u'Which taxonomy id should be use to display category'),
        description=_(u'Please write which taxonomy id should be used.'),
        default=u'',
        required=False,
    )

    item_count_homepage = schema.Int(
        title=_(
            u'label_item_count_homepage',
            default=u'Item count for homepage'),
        description=_(u'Number of items that will show up in one homepage.'),
        required=False,
        default=8,
        min=1,
        max=30,
    )

    hide_title = schema.Bool(
        title=_(u'Hide title'),
        description=_(u'Do you want to hide title on index view ?'),
        required=False,
        default=False
    )

    hide_see_all_link = schema.Bool(
        title=_(u'Hide see all link'),
        description=_(u'Do you want to hide see all link on index view ?'),
        required=False,
        default=False
    )

    hide_date = schema.Bool(
        title=_(u'Hide date'),
        description=_(u'Do you want to hide date on index view ?'),
        required=False,
        default=False
    )

    show_day_and_month = schema.Bool(
        title=_(u'Show day and month'),
        description=_(
            u'Do you want to show day and month on index view ?'),
        required=False,
        default=False
    )

    show_lead_image = schema.Bool(
        title=_(u'Show lead image'),
        description=_(
            u'Do you want to show lead image on index view ?'),
        required=False,
        default=True
    )

    show_descriptions = schema.Bool(
        title=_(u'Show items descriptions'),
        description=_(
            u'Do you want to show items descriptions on index view ?'),
        required=False,
        default=False
    )

    use_new_template = schema.Bool(
        title=_(u'Use new template'),
        description=_(u'Do you want to use the new (experimental) template ?'),
        required=False,
        default=False
    )

    show_event_category_below_image = schema.Bool(
        title=_(u'Event category below image'),
        description=_(u'Show event category below image, with date and title'),
        required=False,
        default=False
    )

    use_center_mode = schema.Bool(
        title=_(u"Center mode (slick)"),
        description=_(u'Use center mode for slick '),
        required=False,
        default=False,
    )

    center_padding = schema.Int(
        title=_(u"Center padding (slick)"),
        description=_(u'Set padding on the sides for center mode '),
        required=False,
        default=50,
    )
    
    slide_to_scroll = schema.Int(
        title=_(u"Slide to scroll (slick)"),
        description=_(u'Number of slide to scroll '),
        required=False,
        default=1,
        min=1,
        max=10,
    )

    autoplay_mode = schema.Bool(
        title=_(u"Autoplay mode (slick)"),
        description=_(u'Use autoplay mode to activate automatic scrolling'),
        required=False,
        default=False,
    )

    autoplay_speed = schema.Int(
        title=_(u"Autoplay speed (slick)"),
        description=_(u'Use autoplay_speed to define scrolling speed '),
        required=False,
        default=4000,
    )

    show_arrows = schema.Bool(
        title=_(u"Show arrows (slick)"),
        description=_(u'Ative to show the arrows'),
        required=False,
        default=True,
    )

    show_dots = schema.Bool(
        title=_(u"Show dots (slick)"),
        description=_(u'Ative to show the dots'),
        required=False,
        default=True,
    )
    
    fade = schema.Bool(
        title=_(u"Fade mode (slick)"),
        description=_(u'Ative to Fade mode'),
        required=False,
        default=False,
    )

    speed = schema.Int(
        title=_(u"Slide Speed (slick)"),
        description=_(u'Choose the scroll speed'),
        required=False,
        default=300,
    )

    breakpoint_full = schema.Int(
        title=_(u"Breakpoint full (slick)"),
        description=_(u'What size to display in breakpoint_full'),
        required=False,
        default=1024,
    )

    slides_to_show_full = schema.Int(
        title=_(u"slidesToShow_full (slick)"),
        description=_(u"how many element to display in breakpoint_full"),
        required=False,
        default=3,
    )

    slides_to_scroll_full = schema.Int(
        title=_(u"slidesToScroll_full (slick)"),
        description=_(u"how many element scroll in breakpoint_full"),
        required=False,
        default=1,
    )

    breakpoint_medium = schema.Int(
        title=_(u"Breakpoint medium (slick)"),
        description=_(u'What size to display in breakpoint_medium'),
        required=False,
        default=600,
    )

    slides_to_show_medium = schema.Int(
        title=_(u"slidesToShow_medium (slick)"),
        description=_(u"how many element to display in breakpoint_medium"),
        required=False,
        default=2,
    )
    slides_to_scroll_medium = schema.Int(
        title=_(u"slidesToScroll_medium (slick)"),
        description=_(u"how many element scroll in breakpoint_medium"),
        required=False,
        default=1,
    )

    breakpoint_small = schema.Int(
        title=_(u"Breakpoint small (slick)"),
        description=_(u'What size to display in breakpoint_small'),
        required=False,
        default=480,
    )

    slides_to_show_small = schema.Int(
        title=_(u"slidesToShow_small (slick)"),
        description=_(u"how many element to display in breakpoint_small"),
        required=False,
        default=1,
    )
    slides_to_scroll_small = schema.Int(
        title=_(u"slidesToScroll_small (slick)"),
        description=_(u"how many element scroll in breakpoint_small"),
        required=False,
        default=1,
    )



# -*- coding: utf-8 -*-
from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserSubMenuItem


class IConfigurationsSubMenuItem(IBrowserSubMenuItem):
    """The menu item linking to the cpskin configurations menu.
    """


class IConfigurationsMenu(IBrowserMenu):
    """The configurations menu.

    This gets its menu items from portal_actions.
    """



# -*- coding: utf-8 -*-
from cpskin.minisite.browser.interfaces import IHNavigationActivated
from cpskin.minisite.interfaces import IInMinisiteBase
from cpskin.minisite.utils import get_minisite_navigation_level
from cpskin.minisite.utils import get_minisite_object
from plone import api
from plone.app.layout.navigation.interfaces import INavigationQueryBuilder
from plone.app.layout.navigation.interfaces import INavtreeStrategy
from plone.app.portlets.portlets.navigation import getRootPath
from plone.app.portlets.portlets.navigation import INavigationPortlet
from plone.app.portlets.portlets.navigation import NavtreeStrategy
from plone.app.portlets.portlets.navigation import QueryBuilder
from plone.app.portlets.portlets.navigation import Renderer
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface


HAS_MENU = False
try:
    from cpskin.menu.interfaces import IFourthLevelNavigation
    HAS_MENU = True
except ImportError:
    pass


def calculateTopLevel(context, portlet, request=None):
    """Calculate top level of navigation menu to take care of 4th level menu
    NB : IFourthLevelNavigation is activated on the third level folder
    """
    if portlet.topLevel != 3 or not HAS_MENU:
        return portlet.topLevel
    portal = api.portal.get()
    contextPhyPath = context.getPhysicalPath()
    portalPhyPath = portal.getPhysicalPath()
    path = [elem for elem in list(contextPhyPath) if elem not in list(portalPhyPath)]  # noqa
    depth = len(path)
    if depth >= 3 and 'dexterity-types' not in path:
        subLevels = depth - 3
        if subLevels:
            thirdLevelPath = '/'.join(contextPhyPath[:-subLevels])
        else:
            thirdLevelPath = '/'.join(contextPhyPath)
        thirdLevelFolder = portal.unrestrictedTraverse(thirdLevelPath)
        if IFourthLevelNavigation.providedBy(thirdLevelFolder):
            return 4
        if request:
            if IInMinisiteBase.providedBy(request):
                minisite_obj = get_minisite_object(request)
                if minisite_obj and IHNavigationActivated.providedBy(minisite_obj):  # noqa
                    return get_minisite_navigation_level(minisite_obj) + 1
    return portlet.topLevel


class CPSkinQueryBuilder(QueryBuilder):
    implements(INavigationQueryBuilder)
    adapts(Interface, INavigationPortlet)

    def __call__(self):
        topLevel = calculateTopLevel(self.context, self.portlet)
        self.query['path']['navtree_start'] = topLevel + 1
        return self.query


class CPSkinNavtreeStrategy(NavtreeStrategy):
    implements(INavtreeStrategy)
    adapts(Interface, INavigationPortlet)

    def __init__(self, context, portlet):
        NavtreeStrategy.__init__(self, context, portlet)
        portal_properties = getToolByName(context, 'portal_properties')
        navtree_properties = getattr(portal_properties, 'navtree_properties')
        currentFolderOnly = portlet.currentFolderOnly or \
            navtree_properties.getProperty('currentFolderOnlyInNavtree', False)
        topLevel = calculateTopLevel(context, portlet, context.REQUEST)

        self.rootPath = getRootPath(
            context,
            currentFolderOnly,
            topLevel,
            portlet.root)


class CPSkinRenderer(Renderer):
    _template = ViewPageTemplateFile('navigation.pt')
    recurse = ViewPageTemplateFile('navigation_recurse.pt')

    @memoize
    def getNavRootPath(self):
        currentFolderOnly = self.data.currentFolderOnly or \
            self.properties.getProperty('currentFolderOnlyInNavtree', False)

        topLevel = calculateTopLevel(self.context, self.data, self.request)
        root = self.data.root
        if isinstance(root, unicode):
            root = str(root)

        return getRootPath(self.context, currentFolderOnly, topLevel, root)

    def createNavTree(self):
        data = self.getNavTree()
        for item in data.get('children'):
            if item.get('portal_type') == 'Link':
                current_user = api.user.get_current()
                obj_link = item.get('item').getObject()
                can_edit = current_user.has_permission('Edit', obj_link)
                item['target_blank'] = '_blank' if getattr(obj_link, 'target_blank', False) and not can_edit else '_self'
        bottomLevel = self.data.bottomLevel or 0

        if bottomLevel < 0:
            # Special case where navigation tree depth is negative
            # meaning that the admin does not want the listing to be displayed
            return self.recurse([], level=1, bottomLevel=bottomLevel)
        else:
            return self.recurse(children=data.get('children', []), level=1, bottomLevel=bottomLevel)



# -*- coding: utf-8 -*-
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING

import unittest


class TestPermissions(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_portlets_permissions(self):
        portal = self.layer['portal']
        permission = 'Portlets: Manage portlets'
        roles = [r['name'] for r in portal.rolesOfPermission(permission) if r['selected']]  # noqa
        self.assertEqual(
            roles,
            ['Editor', 'Manager', 'Portlets Manager', 'Site Administrator']
        )

        permission = 'CPSkin: Edit keywords'
        roles = [r['name'] for r in portal.rolesOfPermission(permission) if r['selected']]  # noqa
        self.assertEqual(
            roles,
            ['Manager', 'Portlets Manager', 'Site Administrator']
        )

        permission = 'CPSkin: Site administrator'
        roles = [r['name'] for r in portal.rolesOfPermission(permission) if r['selected']]  # noqa
        self.assertEqual(roles, ['Manager', 'Site Administrator'])



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone.z3cform.textlines.textlines import TextLinesWidget
from z3c.form import util
from z3c.form.converter import BaseDataConverter
from z3c.form.widget import FieldWidget
from zope.component import adapts
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import ISequence

import six


class IMultiLineWidget(Interface):
    pass


@implementer(IMultiLineWidget)
class MultiLineWidget(TextLinesWidget):
    pass


class MultiLineDataConverter(BaseDataConverter):
    adapts(ISequence, IMultiLineWidget)

    def toWidgetValue(self, value):
        """See interfaces.IDataConverter"""
        if value is self.field.missing_value:
            return ['']
        if isinstance(value, six.string_types):
            return [util.toUnicode(value)]
        return [util.toUnicode(v) for v in value if v]

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        if self._strip_value and isinstance(value, six.string_types):
            value = [value.strip()]

        if value == u'' or value == []:
            return self.field.missing_value

        return [v for v in value if v]


def multiline_field_widget(field, request):
    return FieldWidget(field, MultiLineWidget(request))



# -*- coding: utf-8 -*-

from plone import api
from plone.app.contenttypes.browser.file import FileView as BaseFileView


class FileView(BaseFileView):

    def __call__(self):
        context = self.context
        if api.user.is_anonymous():
            pdf_url = context.absolute_url()
            self.request.response.redirect(pdf_url)
            return ''
        return super(FileView, self).__call__()



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from collective.navigationtoggle.interfaces import INavigationToggleSettings
from cpskin.core.browser.interfaces import INavigationToggleView
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import IFolderish
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implements


class NavigationToggleView(BrowserView):
    """Navigation toggle activation helper view"""
    implements(INavigationToggleView)

    def _redirect(self, msg=''):
        if self.request:
            if msg:
                api.portal.show_message(message=msg,
                                        request=self.request,
                                        type='info')
            self.request.response.redirect(self.context.absolute_url())
        return msg

    def _get_real_context(self):
        context = self.context
        plone_view = getMultiAdapter((context, self.request), name="plone")
        if plone_view.isDefaultPageInFolder():
            context = aq_parent(context)
        context = aq_inner(context)
        return context

    def relative_path(self, obj):
        portal_url = api.portal.get().absolute_url()
        relative_path = obj.absolute_url().replace(portal_url, '')
        if not relative_path.startswith('/'):
            relative_path = '/%s' % relative_path
        return unicode(relative_path)

    @property
    def settings(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INavigationToggleSettings, check=False)
        return settings

    @property
    def is_enabled(self):
        context = self._get_real_context()
        if not IFolderish.providedBy(context):
            return False
        context = self._get_real_context()
        return self.relative_path(context) in self.settings.selectors

    @property
    def can_enable_navigation_toggle(self):
        context = self._get_real_context()
        if not IFolderish.providedBy(context):
            return False
        return not self.is_enabled

    @property
    def can_disable_navigation_toggle(self):
        return self.is_enabled

    def enable_navigation_toggle(self):
        """Enable the navigation toggle on folder"""
        context = self._get_real_context()
        current_paths = self.settings.selectors
        current_paths_list = list(current_paths)
        current_paths_list.append(self.relative_path(context))
        self.settings.selectors = tuple(current_paths_list)
        portal_js = api.portal.get_tool('portal_javascripts')
        portal_js.cookResources()
        self._redirect(_(u'Navigation toggle enabled for folder'))

    def disable_navigation_toggle(self):
        """Disable the navigation toggle on folder"""
        context = self._get_real_context()
        current_paths = self.settings.selectors
        current_paths_list = list(current_paths)
        current_paths_list.remove(self.relative_path(context))
        self.settings.selectors = tuple(current_paths_list)
        portal_js = api.portal.get_tool('portal_javascripts')
        portal_js.cookResources()
        self._redirect(_(u'Navigation toggle disabled for folder'))



# -*- coding: utf-8 -*-
from cpskin.core.interfaces import IAlbumCollection
from cpskin.core.interfaces import IVideoCollection
from cpskin.core.utils import image_scale
from imio.media.browser import utils
from plone import api
from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter

import logging


logger = logging.getLogger('cpskin.core media viewlet')


class MediaViewlet(common.ViewletBase):

    index = ViewPageTemplateFile('media.pt')

    def available(self):
        context = self.context
        media_view = getMultiAdapter((context, self.request),
                                     name='media_activation')
        return media_view.is_enabled

    @property
    def portal_catalog(self):
        return api.portal.get_tool(name='portal_catalog')

    def get_videos_collection(self):
        return self.get_collection(IVideoCollection)

    def get_videos(self):
        videos = []
        collection = self.get_videos_collection()
        if not collection:
            return None
        limit = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_videos')  # noqa
        brains = [b for b in collection.queryCatalog()][:limit]
        for brain in brains:
            video = brain.getObject()
            videos.append(utils.embed(video, self.request))
        return videos

    def get_albums_collection(self):
        return self.get_collection(IAlbumCollection)

    def get_albums(self):
        albums = []
        collection = self.get_albums_collection()
        if not collection:
            logger.debug('{0} has no album collection'.format(self.context))
            return ''
        for gallery_brain in collection.queryCatalog():
            gallery = gallery_brain.getObject()
            imagescale = self.context.unrestrictedTraverse(
                gallery.getPhysicalPath() + ('@@images',))
            if imagescale:
                scale = image_scale(
                    gallery,
                    'image',
                    'multimedia',
                    with_uid=False)
                if not scale:
                    logger.debug(
                        '{0} has no album collection'.format(self.context))
                else:
                    html = '<a href="{0}">'.format(gallery.absolute_url())
                    html += scale.tag()
                    html += '</a>'
                    albums.append(html)
            else:
                logger.debug('{0} has no lead image'.format(
                    gallery_brain.getURL()))
        limit = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_albums')  # noqa
        return albums[:limit]

    def get_collection(self, object_provide):
        queryDict = {}
        queryDict['object_provides'] = object_provide.__identifier__
        queryDict['path'] = {
            'query': '/'.join(self.context.getPhysicalPath()),
            'depth': 1
        }
        queryDict['sort_limit'] = 1
        brains = self.portal_catalog(queryDict)
        if len(brains) == 0:
            return None
        brain = brains[0]
        folder = brain.getObject()
        collection = getattr(folder, 'index', None)
        if not collection:
            return None
        return collection



# -*- coding: utf-8 -*-
from cpskin.core.utils import add_behavior
from plone import api
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneWithPackageLayer
from plone.app.textfield.value import RichTextValue
from plone.testing import z2

import cpskin.core


class CPSkinCorePloneWithPackageLayer(PloneWithPackageLayer):
    """
    """

    def setUpZope(self, app, configurationContext):
        super(CPSkinCorePloneWithPackageLayer, self).setUpZope(
            app,
            configurationContext
        )
        z2.installProduct(app, 'Products.DateRecurringIndex')
        # import plone.app.contenttypes
        # self.loadZCML(package=plone.app.contenttypes)
        # import plone.app.event
        # self.loadZCML(package=plone.app.event)

    def tearDownZope(self, app):
        # Uninstall products installed above
        z2.uninstallProduct(app, 'Products.DateRecurringIndex')

    def setUpPloneSite(self, portal):
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')
        # applyProfile(portal, 'plone.app.contenttypes:plone-content')
        applyProfile(portal, 'cpskin.core:testing')
        richtextvalue = RichTextValue(
            u'Footer static custom content',
            'text/plain',
            'text/html'
        )
        footer_static = portal['footer-static']
        footer_static.text = richtextvalue
        add_behavior('Document', 'cpskin.core.behaviors.metadata.IHiddenTags')
        add_behavior('News Item', 'collective.sticky.behavior.ISticky')
        googleapi = 'collective.geo.settings.interfaces.IGeoSettings.googleapi'
        api.portal.set_registry_record(
            googleapi,
            u'AIzaSyChlyBRkPYJvny6t0mRqawQtQao2cJBvaA')
        fti = api.portal.get_tool('portal_types')['Plone Site']
        fti.allowed_content_types = fti.allowed_content_types + ('directory',)


CPSKIN_CORE_FIXTURE = CPSkinCorePloneWithPackageLayer(
    name='CPSKIN_CORE_FIXTURE',
    zcml_filename='testing.zcml',
    zcml_package=cpskin.core,
    gs_profile_id='cpskin.core:testing')

CPSKIN_CORE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CPSKIN_CORE_FIXTURE,),
    name='cpskin.core:Integration')

CPSKIN_CORE_ROBOT_TESTING = FunctionalTesting(
    bases=(CPSKIN_CORE_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE,
           z2.ZSERVER_FIXTURE),
    name='cpskin.core:Robot')



# -*- coding: utf-8 -*-
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.event.interfaces import IEvent
from plone.namedfile import field as namedfile
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ICpskinEventViewSettings(model.Schema):
    model.fieldset(
        'eventview',
        label=_(
            u'Event view'),
        fields=(
            'image_banner',
        ),
    )

    model.fieldset(
        'eventview',
        label=_(
            u'Event view'),
        fields=(
            'image_header',
        ),
    )

    image_banner = namedfile.NamedBlobImage(
        title=_(u'label_bannerimage', default=u'Banner image'),
        description=_(
            u'help_bannerimage',
            default=u'Image used as banner replacement when viewing this event.<br/>This image can be crop into banner format'
        ),
        required=False,
    )

    image_header = namedfile.NamedBlobImage(
        title=_(u'label_headerimage', default=u'Header image'),
        description=_(
            u'help_headerimage',
            default=u'Image used as header in the event.<br/>This image can be crop into banner_event or banner format if banner_event is not available.'
        ),
        required=False,
    )

@implementer(ICpskinEventViewSettings)
@adapter(IEvent)
class CpskinEventViewSettings(object):

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class CPSkinFooterSitemapViewlet(ViewletBase):
    render = ViewPageTemplateFile('footersitemap.pt')

    def showSiteMap(self):
        return api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.show_footer_sitemap')

    def createSiteMap(self):
        context = aq_inner(self.context)
        # take the 2 first levels of the site that respect the navigation
        # strategy
        portal_catalog = getToolByName(context, 'portal_catalog')
        navtreeProps = getToolByName(
            context, 'portal_properties').navtree_properties
        queryDict = {}
        navigation_root = api.portal.get_navigation_root(context)
        queryDict['path'] = {
            'query': '/'.join(navigation_root.getPhysicalPath()), 'depth': 1}
        if navtreeProps.enable_wf_state_filtering:
            queryDict['review_state'] = navtreeProps.wf_states_to_show
        queryDict['sort_on'] = 'getObjPositionInParent'
        themes = portal_catalog(queryDict)
        res = []
        metaTypesNotToList = navtreeProps.metaTypesNotToList
        idsNotToList = navtreeProps.idsNotToList
        for theme in themes:
            if theme.meta_type not in metaTypesNotToList and \
                    theme.id not in idsNotToList and not theme.exclude_from_nav:
                themeRes = {'theme': theme, 'children': []}
                # do a second catalog_search by theme
                queryDict['path'] = {'query': theme.getPath(), 'depth': 1}
                children = portal_catalog(queryDict)
                for child in children:
                    if child.meta_type not in metaTypesNotToList and \
                            child.id not in idsNotToList and not child.exclude_from_nav:
                        themeRes['children'].append(child)
                res.append(themeRes)
        return res

    def getFooterText(self):
        navigation_root = api.portal.get_navigation_root(self.context)
        footer_static = getattr(navigation_root, 'footer-static', None)
        text = ''
        if footer_static is None:
            return
        if footer_static.Language() == self.context.Language():
            if getattr(footer_static, 'text', None):
                text = footer_static.text.raw
                return text
        if getattr(footer_static, 'getTranslation', None):
            lang = self.context.REQUEST.get('LANGUAGE', 'fr')
            footer_static = footer_static.getTranslation(lang)
        if getattr(footer_static, 'text', None):
            text = footer_static.text.raw
        return text



# -*- coding: utf-8 -*-
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from cpskin.core.interfaces import ICPSkinCoreLayer
from cpskin.locales import CPSkinMessageFactory as _
from Products.Archetypes import public as atapi
from Products.Archetypes.interfaces import IBaseContent
from Products.CMFCore import permissions as zope_permissions
from zope.component import adapter
from zope.interface import implementer

import permissions


class ExtensionStandardTagsField(ExtensionField, atapi.LinesField):
    """Derivate from Archetypes basic LinesField"""


class ExtensionIAmTagsField(ExtensionField, atapi.LinesField):
    """Derivate from Archetypes basic LinesField"""


class ExtensionISearchTagsField(ExtensionField, atapi.LinesField):
    """Derivate from Archetypes basic LinesField"""


class ExtensionHiddenTagsField(ExtensionField, atapi.LinesField):
    """Derivate from Archetypes basic LinesField"""


@adapter(IBaseContent)
@implementer(IOrderableSchemaExtender, IBrowserLayerAwareExtender)
class ContentExtender(object):
    layer = ICPSkinCoreLayer

    fields = [
        ExtensionStandardTagsField(
            'standardTags',
            multiValued=1,
            searchable=False,
            schemata='categorization',
            languageIndependent=True,
            widget=atapi.KeywordWidget(
                label=_(u'label_standard_tags', default=u'Standard Tags'),
                description=_(u'help_standard_tags',
                              default=u'Standard Tags are used for webmaster '
                                      u'organization of content.'),
            ),
            read_permission=zope_permissions.View,
            write_permission=permissions.CPSkinEditKeywords,
        ),
        ExtensionIAmTagsField(
            'iamTags',
            multiValued=1,
            searchable=False,
            schemata='categorization',
            languageIndependent=True,
            widget=atapi.KeywordWidget(
                label=_(u'label_iam_tags', default=u'I Am Tags'),
                description=_(u'help_iam_tags',
                              default=u'I Am Tags are used for webmaster '
                                      u'organization of content.'),
            ),
            read_permission=zope_permissions.View,
            write_permission=permissions.CPSkinEditKeywords,
        ),
        ExtensionISearchTagsField(
            'isearchTags',
            multiValued=1,
            searchable=False,
            schemata='categorization',
            languageIndependent=True,
            widget=atapi.KeywordWidget(
                label=_(u'label_isearch_tags', default=u'I Search Tags'),
                description=_(u'help_isearch_tags',
                              default=u'I Search Tags are used for webmaster '
                                      u'organization of content.'),
            ),
            read_permission=zope_permissions.View,
            write_permission=permissions.CPSkinEditKeywords,
        ),
        ExtensionHiddenTagsField(
            'hiddenTags',
            multiValued=1,
            searchable=False,
            schemata='categorization',
            languageIndependent=True,
            widget=atapi.KeywordWidget(
                label=_(u'label_hidden_tags', default=u'Hidden Tags'),
                description=_(u'help_hidden_tags',
                              default=u'Hidden Tags are used for webmaster '
                                      u'organization of content.'),
            ),
            read_permission=zope_permissions.View,
            write_permission=permissions.CPSkinEditKeywords,
        ),
    ]

    def __init__(self, context):
        self.context = context

    def getOrder(self, schematas):
        """
        Make sure that tags are just after subject in categorization schemata
        """
        if 'subject' in schematas['categorization']:
            insertAfterFieldIndex = schematas['categorization'].index('subject')  # noqa
        else:
            insertAfterFieldIndex = -1

        standardTagsIndex = schematas['categorization'].index('standardTags')
        schematas['categorization'].insert(
                           insertAfterFieldIndex + 1,
                           schematas['categorization'].pop(standardTagsIndex)
                           )
        iAmTagsIndex = schematas['categorization'].index('iamTags')
        schematas['categorization'].insert(
                           insertAfterFieldIndex + 2,
                           schematas['categorization'].pop(iAmTagsIndex)
                           )
        iSearchTagsIndex = schematas['categorization'].index('isearchTags')
        schematas['categorization'].insert(
                           insertAfterFieldIndex + 3,
                           schematas['categorization'].pop(iSearchTagsIndex)
                           )
        hiddenTagsIndex = schematas['categorization'].index('hiddenTags')
        schematas['categorization'].insert(
                           insertAfterFieldIndex + 4,
                           schematas['categorization'].pop(hiddenTagsIndex)
                           )
        return schematas

    def getFields(self):
        return self.fields



# -*- coding: utf-8 -*-
from collective.contact.core.adapters import ContactDetailsVCard
from collective.contact.core.adapters import OrganizationVCard
from collective.contact.core.interfaces import IContactable
from collective.documentgenerator.content.condition import ConfigurablePODTemplateCondition  # noqa
from imio.dashboard.content.pod_template import DashboardPODTemplateCondition as DPTC  # noqa
from Products.CMFPlone.utils import safe_unicode

import vobject


class DashboardPODTemplateCondition(DPTC):

    def evaluate(self):
        return super(ConfigurablePODTemplateCondition, self).evaluate()


def get_vcard(context):
    vcard = vobject.vCard()
    contactable = IContactable(context)
    contact_details = contactable.get_contact_details()

    email = contact_details['email']
    if email:
        vcard.add('email')
        vcard.email.type_param = 'INTERNET'
        vcard.email.value = email

    phones = contact_details['phone']
    if not isinstance(phones, list):
        phones = [phones]
    for phone in phones:
        if phone:
            vcard.add('tel')
            vcard.tel.type_param = 'WORK'
            vcard.tel.value = phone

    cell_phones = contact_details['cell_phone']
    if not isinstance(cell_phones, list):
        cell_phones = [cell_phones]
    for cell_phone in cell_phones:
        if cell_phone:
            vcard.add('tel')
            last_item = len(vcard.tel_list) - 1
            vcard.tel_list[last_item].type_param = 'CELL'
            vcard.tel_list[last_item].value = cell_phone

    im_handle = contact_details['im_handle']
    if im_handle:
        vcard.add('impp')
        vcard.impp.value = im_handle

    address = contact_details['address']

    # if we don't have relevant address information, we don't need address
    if address:
        vcard.add('adr')
        country = safe_unicode(address['country'], encoding='utf8')
        region = safe_unicode(address['region'], encoding='utf8')
        zip_code = safe_unicode(address['zip_code'], encoding='utf8')
        city = safe_unicode(address['city'], encoding='utf8')
        street = safe_unicode(address['street'], encoding='utf8')
        number = safe_unicode(address['number'], encoding='utf8')
        additional = safe_unicode(address['additional_address_details'],
                                  encoding='utf8')
        vcard.adr.value = vobject.vcard.Address(street=street,
                                                city=city,
                                                region=region,
                                                code=zip_code,
                                                country=country,
                                                box=number,
                                                extended=additional)
    return vcard


class CPskinContactDetailsVCard(ContactDetailsVCard):

    def get_vcard(self):
        vcard = get_vcard(self.context)
        vcard.add('fn')
        vcard.fn.value = safe_unicode(self.context.Title())
        vcard.add('n')
        vcard.n.value = vobject.vcard.Name(safe_unicode(self.context.Title()))
        return vcard


class CPskinOrganizationVCard(OrganizationVCard):
    """Override from OrganizationVCard to get multi phones and mutli
    cell_phones on vcard export"""

    def get_vcard(self):
        vcard = get_vcard(self.context)
        vcard.add('kind')
        vcard.kind.value = 'org'

        organization = self.context
        title = safe_unicode(organization.Title(), encoding='utf8')
        vcard.add('n')
        vcard.n.value = vobject.vcard.Name(title)
        vcard.add('fn')
        vcard.fn.value = title
        return vcard



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:copyright: (c) 2015 by Affinitic SPRL
:license: GPL, see LICENCE.txt for more details.
"""
from cpskin.locales import CPSkinMessageFactory as _
from eea.facetednavigation.layout.layout import FacetedLayout
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.schema.vocabulary import SimpleVocabulary


class FacetedLayoutVocabularyFactory(object):

    def __call__(self, context):
        faceted_layout = FacetedLayout(context)
        request = getattr(context, 'REQUEST', getRequest())
        # remove duplicates layouts:
        layouts = list(set(faceted_layout.layouts))
        terms = [
            SimpleVocabulary.createTerm(
                l[0],
                l[0],
                translate(_(l[1]), context=request),
            ) for l in layouts]
        return SimpleVocabulary(terms)


FacetedLayoutVocabulary = FacetedLayoutVocabularyFactory()



# -*- coding: utf-8 -*-
from plone.app.search.browser import Search


class CpskinSearch(Search):

    def filter_query(self, query):
        query = super(CpskinSearch, self).filter_query(query)
        if query is None:
            return {}
        if 'SearchableText' in query.keys():
            searchable_text = query.get('SearchableText', '')
            if searchable_text:
                searchable_text = '{0}*'.format(searchable_text)
                query['SearchableText'] = searchable_text
        return query



# -*- coding: utf-8 -*-
from plone import api
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


class IPhotoPortlet(IPortletDataProvider):
    """
    Photo portlet interface
    """


class Assignment(base.Assignment):
    implements(IPhotoPortlet)


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('photo.pt')

    @property
    def available(self):
        """
        Portlet should be available from the 3 level of navigation
        """
        contextPhyPath = self.context.getPhysicalPath()
        portalPhyPath = api.portal.get().getPhysicalPath()
        path = [elem for elem in list(contextPhyPath) if elem not in list(portalPhyPath)]  # noqa
        depth = len(path)
        if depth < 2:
            return False
        return True

    def get_visuel(self):
        visuel = getattr(self.context, 'visuel.png', None)
        if visuel is not None:
            visuel = visuel.aq_inner
            return visuel.absolute_url()
        return ''


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.contact.core.browser.contactable import ContactDetails
from collective.contact.core.interfaces  import IContactable
from collective.geo.geographer.geoview import GeoView
from cpskin.core import utils
from plone import api
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ua_parser import user_agent_parser

import os
import six


class ContactDetailsView(BrowserView, ContactDetails):

    template_path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'templates',
        'address.pt',
    )

    def __call__(self):
        self.update()
        return super(ContactDetailsView, self).__call__()

    def update(self):
        fallback = self.is_contact_core_fallback()
        contactable = IContactable(self.context)
        #noFallbackContactDetails or fallback.
        self.contact_details = contactable.get_contact_details(fallback=fallback)

    def is_contact_core_fallback(self):
        portal_registry = api.portal.get_tool('portal_registry')
        return portal_registry['cpskin.core.interfaces.ICPSkinSettings.person_contact_core_fallback']  # noqa


    @property
    def phones(self):
        phones = self.contact_details.get('phone', [])
        if isinstance(phones, six.string_types):
            phones = [phones]
        return [utils.format_phone(v) for v in phones]

    @property
    def cell_phones(self):
        cell_phones = self.contact_details.get('cell_phone', [])
        if isinstance(cell_phones, six.string_types):
            cell_phones = [cell_phones]
        return [utils.format_phone(v) for v in cell_phones]

    @property
    def fax(self):
        fax = self.contact_details.get('fax')
        if fax:
            return utils.format_phone(fax)

    def geo_converter(self, longitude, latitude):
        infos = user_agent_parser.Parse(self.request.get('HTTP_USER_AGENT'))
        links = {
            'Mac OS X': 'maps:?q={longitude},{latitude}',
            'iOS': 'maps:?q={longitude},{latitude}',
            'Android': 'geo:?q={longitude},{latitude}',
            'Windows Phone': 'maps:?q={longitude},{latitude}',
            'Windows 10': 'maps:?q={longitude},{latitude}',
            'Windows': 'maps:?q={longitude},{latitude}',
        }
        if infos['os']['family'] not in links:
            return '#map'
        return links[infos['os']['family']].format(
            longitude=longitude,
            latitude=latitude,
        )

    @property
    def geo_link(self):
        geo_view = GeoView(self.context, self.request)
        if geo_view.hasCoordinates():
            coordinates = geo_view.getCoordinates()
            return self.geo_converter(
                coordinates[1][1],
                coordinates[1][0],
            )

    def render_address(self):
        link = self.geo_link
        template = ViewPageTemplateFile(self.template_path)
        return template(self, self.contact_details['address'], link)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from cpskin.core.browser.interfaces import IBannerActivationView
from cpskin.core.interfaces import IBannerActivated
from cpskin.core.interfaces import ILocalBannerActivated
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import noLongerProvides


class BannerActivationView(BrowserView):
    """
    Banner activation helper view
    """
    implements(IBannerActivationView)

    def _redirect(self, msg=''):
        if self.request:
            if msg:
                api.portal.show_message(message=msg,
                                        request=self.request,
                                        type='info')
            self.request.response.redirect(self.context.absolute_url())
        return msg

    def _get_real_context(self):
        context = self.context
        plone_view = getMultiAdapter((context, self.request), name="plone")
        if plone_view.isDefaultPageInFolder():
            context = aq_parent(context)
        context = aq_inner(context)
        return context

    @property
    def is_enabled(self):
        context = self._get_real_context()
        return self.banner_root is not None or ILocalBannerActivated.providedBy(context)

    @property
    def banner_root(self):
        context = self._get_real_context()
        portal = api.portal.get()
        obj = context
        while obj != portal:
            if IBannerActivated.providedBy(obj):
                return obj
            obj = aq_parent(obj)
        if IBannerActivated.providedBy(obj):
            return obj
        return None

    @property
    def can_enable_banner(self):
        return not self.is_enabled

    @property
    def can_disable_banner(self):
        context = self._get_real_context()
        return(IBannerActivated.providedBy(context))

    def enable_banner(self):
        """ Enable the banner """
        context = self._get_real_context()
        alsoProvides(context, IBannerActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Banner enabled on content and sub-contents'))

    def disable_banner(self):
        """ Disable the banner """
        context = self._get_real_context()
        noLongerProvides(context, IBannerActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Banner disabled for content and sub-contents'))

    @property
    def can_enable_local_banner(self):
        return not self.can_disable_local_banner

    @property
    def can_disable_local_banner(self):
        context = self._get_real_context()
        return(ILocalBannerActivated.providedBy(context))

    def enable_local_banner(self):
        """ Enable the banner """
        context = self._get_real_context()
        alsoProvides(context, ILocalBannerActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Banner enabled on content'))

    def disable_local_banner(self):
        """ Disable the banner """
        context = self._get_real_context()
        noLongerProvides(context, ILocalBannerActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Banner disabled for content'))



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:copyright: (c) 2015 by Affinitic SPRL
:license: GPL, see LICENCE.txt for more details.
"""
from cpskin.locales import CPSkinMessageFactory as _
from eea.facetednavigation import EEAMessageFactory as EEAMF
from eea.facetednavigation.widgets.widget import Widget as AbstractWidget
from Products.Archetypes.public import MultiSelectionWidget
from Products.Archetypes.public import Schema
from Products.Archetypes.public import StringField
from Products.Archetypes.public import StringWidget
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import logging


logger = logging.getLogger('eea.facetednavigation.widgets.portlet')

EditSchema = Schema((
    StringField(
        'values',
        schemata='default',
        required=True,
        vocabulary_factory='cpskin.vocabularies.faceted_layout',
        widget=MultiSelectionWidget(
            label=_(u'Views'),
            i18n_domain='cpskin',
        ),
    ),
    StringField(
        'default',
        schemata='default',
        widget=StringWidget(
            size=25,
            label=EEAMF(u'Default value'),
            description=EEAMF(u'Default selected item'),
            i18n_domain='eea',
        )
    ),
))


class Widget(AbstractWidget):
    """ Widget
    """
    # Widget properties
    widget_type = 'layout'
    widget_label = _('Layout selection')
    view_js = '++resource++cpskin.faceted.widgets.layout.view.js'
    edit_js = '++resource++cpskin.faceted.widgets.layout.edit.js'
    view_css = '++resource++cpskin.faceted.widgets.layout.view.css'
    edit_css = '++resource++cpskin.faceted.widgets.layout.edit.css'

    index = ZopeTwoPageTemplateFile('layout.pt', globals())
    edit_schema = AbstractWidget.edit_schema.copy() + EditSchema
    edit_schema['title'].default = 'Layout'

    def __init__(self, *args, **kwargs):
        super(Widget, self).__init__(*args, **kwargs)
        self.voc = getUtility(
            IVocabularyFactory,
            name='cpskin.vocabularies.faceted_layout',
        )(self.context)

    @property
    def values(self):
        return [self.voc.getTerm(v) for v in self.data.get('values', [])]



# -*- coding: utf-8 -*-
from AccessControl.SecurityInfo import ModuleSecurityInfo
from Products.CMFCore.permissions import setDefaultRoles


security = ModuleSecurityInfo('cpskin.core.permissions')

security.declarePublic('CPSkinSiteAdministrator')
CPSkinSiteAdministrator = 'CPSkin: Site administrator'
setDefaultRoles(CPSkinSiteAdministrator, ('Site Administrator',
                                          'Manager'))

security.declarePublic('CPSkinEditKeywords')
CPSkinEditKeywords = 'CPSkin: Edit keywords'
setDefaultRoles(CPSkinEditKeywords, ('Site Administrator',
                                     'Manager',
                                     'Portlets Manager'))



# -*- coding: utf-8 -*-
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class ICpskinDirectoryViewSettings(model.Schema):
    model.fieldset(
        'directoryview',
        label=_(
            u'Directory view'),
        fields=(
            'taxonomy_category',
            'show_organization_images',
            'organization_image_scale',
        ),
    )

    taxonomy_category = schema.TextLine(
        title=_(u'Which taxonomy id should be use to display category'),
        description=_(u'Please write which taxonomy id should be used.'),
        default=u'',
        required=False,
    )

    show_organization_images = schema.Bool(
        title=_(u'Show organizations images'),
        description=_(u'Do you want to show images for organizations preview in directory ?'),  # noqa
        required=False,
        default=False,
    )

    organization_image_scale = schema.Choice(
        title=_(u'Which image scale use for organizations preview in directory ?'),  # noqa
        description=_(u'Please select the scale.'),
        required=False,
        default='mini',
        vocabulary=u'plone.app.vocabularies.ImagesScales'
    )



# -*- coding: utf-8 -*-
from collective.anysurfer.interfaces import ILayerSpecific as IAnysurferLayer
from cpskin.locales import CPSkinMessageFactory as _
from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from sc.social.like.interfaces import ISocialLikeLayer
from zope import schema
from zope.interface import Interface


class ICPSkinCoreLayer(ISocialLikeLayer, IPloneAppContenttypesLayer, IAnysurferLayer):
    """
    Marker interface that defines a ZTK browser layer.
    """


class ICPSkinCoreWithMembersLayer(Interface):
    """
    Marker interface that defines a ZTK browser layer.
    """


class IBannerActivated(Interface):
    """
    Marker interface to enable / disable banner viewlet
    """


class ILocalBannerActivated(Interface):
    """
    Marker interface to enable / disable banner viewlet
    """


class IMediaActivated(Interface):
    """
    Marker interface to enable / disable (multi)media viewlet
    """


class IFolderViewSelectedContent(Interface):
    """
    Marker interface to add / remove content to / from folder view
    """


class IFolderViewWithBigImages(Interface):
    """
    Marker interface to use big images on folder view
    """


class IElectedContentForTopMenu(Interface):
    """
    Marker interface to select content for action top menu
    """


class ICPSkinSettings(Interface):
    """
    Settings for CPSkin
    """
    load_page_menu = schema.Bool(
        title=_(u'Load page menu'),
        description=_(u'Is level 1 menu load page at click?'),
        required=False,
        default=False
    )

    sub_menu_persistence = schema.Bool(
        title=_(u'Sub menu persistence'),
        description=_(u'Is level 2 menu persist?'),
        required=False,
        default=False
    )

    auto_play_slider = schema.Bool(
        title=_(u'Auto play slider'),
        description=_(u'Is the front page slider automatically play?'),
        required=False,
        default=False
    )

    slider_timer = schema.Int(
        title=_(u'Slider timer'),
        description=_(u'Number of milliseconds between each transition.'),
        required=False,
        default=5000
    )

    city_name = schema.TextLine(
        title=_(u'City name'),
        description=_(u'Name of city is used in some templates.'),
        required=True,
        default=u'City name'
    )

    contents_in_action_menu = schema.Tuple(
        title=_(u'Content to show in special action menu (top)'),
        description=_(u'Please select which contents should be taken to this menu.'),  # noqa
        required=False,
        value_type=schema.Choice(
            vocabulary=u'cpskin.core.vocabularies.action_menu_eligible'
        )
    )

    show_leadimage_in_action_menu = schema.Bool(
        title=_(u'Show leadimage in action menu'),
        description=_(u'Show leadimage (if any) in the top action menu with content selected in the field before.'),  # noqa
        required=False,
        default=False
    )

    person_contact_core_fallback = schema.Bool(
        title=_(u'Contact properties fallback?'),
        description=_(u'Do you want than contact properties fallback? Sample :If no tel on a person, so we get phone thanks to person\'f function.'),  # noqa
        required=False,
        default=False
    )

    show_slogan = schema.Bool(
        title=_(u'Show slogan with banner'),
        description=_(u'Show banner title and description as homepage slogan.'),  # noqa
        required=False,
        default=False
    )

    show_portlets_in_right_actions_panel = schema.Bool(
        title=_(u'Show (right) portlets in right actions panel'),
        description=_(u'Show (right) portlets (if any) in right actions panel after related contents.'),  # noqa
        required=False,
        default=False
    )

    media_viewlet_visible_albums = schema.Int(
        title=_(u'Viewlet media : Visible albums'),
        description=_(u'Number of visible albums on media viewlet.'),
        required=False,
        min=0,
        default=5
    )

    media_viewlet_visible_videos = schema.Int(
        title=_(u'Viewlet media : Visible videos'),
        description=_(u'Number of visible videos on media viewlet.'),
        required=False,
        min=0,
        default=2
    )

    show_description_on_themes = schema.Bool(
        title=_(u'Show description on themes'),
        description=_(u'Add content description after every portal tab menu title.'),
        required=False,
        default=False
    )

    search_position = schema.Choice(
        title=_(u'Search position'),
        description=_(u'Search box position in eligible themes.'),
        required=True,
        values=[
            u'default_position',
            u'always_in_navigation',
            u'always_in_actions',
        ],
        default=u'default_position'
    )

    collapse_minisite_menu = schema.Bool(
        title=_(u'Collapse menu on minisites'),
        description=_(u'Automatically collapse portal main menu on minisites.'),  # noqa
        required=False,
        default=False
    )

    show_footer_sitemap = schema.Bool(
        title=_(u'Show footer sitemap'),
        description=_(u'Automatically generate sitemap footer.'),
        required=False,
        default=True
    )

    header_class = schema.Choice(
        title=_(u'Header related class'),
        description=_(u'CSS header class that will be applied to body.'),
        required=False,
        values=[
            u'header-1',
            u'header-2',
            u'header-3',
            u'header-4',
        ],
        default=None
    )

    columns_class = schema.Choice(
        title=_(u'Content columns related class'),
        description=_(u'CSS content columns class that will be applied to body.'),
        required=False,
        values=[
            u'content-1',
            u'content-2',
            u'content-3',
            u'content-4',
        ],
        default=None
    )

    navigation_class = schema.Choice(
        title=_(u'Navigation related class'),
        description=_(u'CSS navigation class that will be applied to body.'),
        required=False,
        values=[
            u'navigation-1',
            u'navigation-2',
            u'navigation-3',
            u'navigation-4',
        ],
        default=None
    )

    footer_class = schema.Choice(
        title=_(u'Footer related class'),
        description=_(u'CSS footer class that will be applied to body.'),
        required=False,
        values=[
            u'footer-1',
            u'footer-2',
            u'footer-3',
            u'footer-4',
        ],
        default=None
    )

    indexed_taxonomies = schema.Text(
        title=_(u'Taxonomies to index'),
        description=_(u'List of taxonomy IDs (one per line) that should be indexed.'),
        required=False,
        default=u'types_activites\n'
    )

    use_slick = schema.Bool(
        title=_(u'Use slick for slider'),
        description=_(u'Do you want to use slick instead of flexslider ?'),
        required=False,
        default=False
    )


class IVideoCollection(Interface):
    """
    Marker interface for video collection used to viewlet multimedia
    """


class IAlbumCollection(Interface):
    """
    Marker interface for album collection used to viewlet multimedia
    """



# -*- coding: utf-8 -*-
from plone.app.querystring.registryreader import QuerystringRegistryReader


class CpskinQuerystringRegistryReader(QuerystringRegistryReader):
    """Adapts a registry object to parse the querystring data."""

    def getVocabularyValues(self, values):
        """Get all vocabulary values if a vocabulary is defined"""
        values = super(CpskinQuerystringRegistryReader,
                       self).getVocabularyValues(values)
        cpskin_indexes = ['hiddenTags',
                          'isearchTags', 'iamTags', 'standardTags']
        values.get(self.prefix + '.field')
        for cpskin_index in cpskin_indexes:
            index = values.get(self.prefix + '.field').get(cpskin_index, None)
            if index is None:
                continue
            for value in index.values():
                if isinstance(value, dict):
                    i18n_value = dict((k.encode('utf8'), v)
                                      for k, v in value.items())
                    values.get(self.prefix + '.field').get(
                        cpskin_index)['values'] = i18n_value
        return values



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective.contact.core.browser.address import get_address
from collective.geo.behaviour.interfaces import ICoordinates
from collective.geo.mapwidget.interfaces import IGeoCoder
from geopy import geocoders
from geopy.exc import GeocoderQueryError
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.dexterity.interfaces import IDexterityFTI
from plone.namedfile.file import NamedBlobImage
from Products.CMFCore.interfaces import ISiteRoot
from zope.annotation.interfaces import IAnnotations
from zope.component import queryUtility
from zope.interface import implements

import logging
import os
import phonenumbers


logger = logging.getLogger('cpskin.core.utils')


def safe_utf8(s):
    """Transform unicode to utf8"""
    if isinstance(s, unicode):
        s = s.encode('utf8')
    return s


def safe_unicode(s):
    """Decode from utf8"""
    if isinstance(s, str):
        s = s.decode('utf8')
    return s


def publish_content(content):
    """Publish an object and hide it for cpskin."""
    wftool = api.portal.get_tool('portal_workflow')
    if wftool.getInfoFor(content, 'review_state') != 'published':
        actions = [a.get('id') for a in wftool.listActions(object=content)]
        if api.content.get_state(obj=content) != 'published_and_hidden':
            # we need to handle both workflows
            if 'publish_and_hide' in actions:
                wftool.doActionFor(content, 'publish_and_hide')
            elif 'publish' in actions:
                wftool.doActionFor(content, 'publish')


def set_exclude_from_nav(obj):
    # dexterity with exludefromnav behavior
    obj.exclude_from_nav = True


def add_keyword(obj, tag_id='hiddenTags', tag_value=[]):
    """Add a keyword to a object."""
    old_value = getattr(obj, tag_id)
    if not old_value:
        values = tag_value
    else:
        values = old_value + tag_value
    setattr(obj, tag_id, values)
    obj.reindexObject()


def has_behavior(type_name, behavior_name):
    """Check if a behavior is on portal_type"""
    fti = queryUtility(IDexterityFTI, name=type_name)
    if not fti:
        return
    behaviors = list(fti.behaviors)
    if behavior_name not in behaviors:
        return False
    else:
        return True


def add_behavior(type_name, behavior_name):
    """Add a behavior to a type"""
    fti = queryUtility(IDexterityFTI, name=type_name)
    if not fti:
        return
    behaviors = list(fti.behaviors)
    if behavior_name not in behaviors:
        behaviors.append(behavior_name)
        fti._updateProperty('behaviors', tuple(behaviors))


def remove_behavior(type_name, behavior_name):
    """Add a behavior to a type"""
    fti = queryUtility(IDexterityFTI, name=type_name)
    if not fti:
        return
    behaviors = list(fti.behaviors)
    if behavior_name in behaviors:
        behaviors.remove(behavior_name)
        fti._updateProperty('behaviors', tuple(behaviors))


def add_leadimage_from_file(container, file_name,
                            folder_name='data', image_field='image'):
    """Add leadimage from a file from a folder"""
    if not container:
        container = api.portal.get()
    data_path = os.path.join(os.path.dirname(__file__), folder_name)
    file_path = os.path.join(data_path, file_name)
    if not getattr(aq_base(container), file_name, False):
        namedblobimage = NamedBlobImage(
            data=open(file_path, 'r').read(),
            filename=unicode(file_name)
        )
        image_container = container
        if not INavigationRoot.providedBy(container) and \
                not ISiteRoot.providedBy(container):
            image_container = container.aq_parent

        image = api.content.create(type='Image',
                                   title=file_name,
                                   image=namedblobimage,
                                   container=image_container)
        image.setTitle(file_name)
        image.reindexObject()
        setattr(container, image_field, namedblobimage)


def has_crop(obj, fieldname, scale):
    crops = IAnnotations(obj).get(PAI_STORAGE_KEY)
    if not crops:
        return False
    return '{0:s}_{1:s}'.format(fieldname, scale) in crops


def image_scale(obj, css_class, default_scale, generate_tag=True, with_uid=True):
    images = obj.restrictedTraverse('@@images')
    if obj.portal_type in ['organization', 'person']:
        if getattr(images, 'logo', False):
            image_field_id = 'logo'
            image = images.scale(image_field_id, scale=default_scale)
        else:
            image = None
    else:
        image_field_id = 'image'
        image = images.scale(image_field_id, scale=default_scale)
    if not image:
        return False
    if not generate_tag:
        return image
    if not with_uid:
        image_path = '{0}/@@images/{1}/{2}'.format(
            obj.absolute_url(),
            image_field_id,
            default_scale)
        image.url = image_path
        return image
    return image.tag(css_class=css_class) if image.tag() else ''


# --------------- Address ---------------

def get_lat_lng_from_address(address):
    """Return tuple with status and geocoder object
       0: error, 1: success, 2: not found, 3: unexpected error
    """
    geocoder = geocoders.Nominatim(
        user_agent='{0}-cpskinapp'.format(api.portal.get().id),
        timeout=10,
    )
    geocode = geocoder.geocode(safe_utf8(address))
    return (1, geocode)


def get_address_from_obj(obj):
    # Event
    loc = getattr(obj, 'location', '')
    if loc:
        return obj.location
    # old card
    if obj.portal_type == 'collective.directory.card':
        street = get_field(obj, 'address')
        zip_code = get_field(obj, 'zip_code')
        city = get_field(obj, 'city')
        address = '{0} {1} {2}'.format(
            street, zip_code, city
        )
        return address

    # collective.contact.core
    dict_address = get_address(obj)
    street = safe_utf8(dict_address.get('street'))
    number = safe_utf8(dict_address.get('number'))
    zip_code = safe_utf8(dict_address.get('zip_code'))
    city = safe_utf8(dict_address.get('city'))
    if street and city:
        address = '{0} {1} {2} {3}'.format(
            number, street, zip_code, city
        )
    else:
        return ''
    return address


def get_field(obj, field_name):
    value = getattr(obj, field_name, '')
    if isinstance(value, int):
        return str(value).encode('utf8')
    if value:
        return value.encode('utf8')
    return ''


def has_lat_lng(obj):
    try:
        if ICoordinates(obj).coordinates:
            return True
    except:  # noqa
        return False
    return False


def set_coord(obj, request):
    if not has_behavior(obj.portal_type, ICoordinates.__identifier__):
        return
    address = get_address_from_obj(obj)
    if address:
        status, geocode = get_lat_lng_from_address(address)
        if status in [0, 2, 3]:
            # 0: stop if limit is 'OVER_QUERY_LIMIT'
            # 2: not found
            # 3: unexpected error
            api.portal.show_message(message=geocode, request=request)
            logger.info(geocode)
        else:
            if geocode and geocode.longitude and geocode.latitude:
                coord = u'POINT({0} {1})'.format(
                    geocode.longitude, geocode.latitude)
                ICoordinates(obj).coordinates = coord
                obj.reindexObject(
                    idxs=['zgeo_geometry', 'collective_geo_styles'])
                path = '/'.join(obj.getPhysicalPath())
                message = 'lat lng of {0} updated'.format(path)
                logger.info(message)
                return message
    else:
        message = 'No address for {0}'.format(
            obj.absolute_url(),
        )
        api.portal.show_message(message=message, request=request)
        logger.info(message)


def format_phone(value):
    try:
        number = phonenumbers.parse(value, 'BE')
    except phonenumbers.NumberParseException:
        return {'raw': value, 'formated': value}
    formated_value = phonenumbers.format_number(
        number,
        phonenumbers.PhoneNumberFormat.INTERNATIONAL,
    )
    country_code = str(number.country_code)
    country_code_with_plus = '+{0}'.format(country_code)
    return {
        'raw': formated_value,
        'formated': formated_value.replace(
            country_code_with_plus,
            '{0} (0)'.format(country_code_with_plus),
        ),
    }


def set_plonecustom_last():
    portal_css = api.portal.get_tool('portal_css')
    resources = list(portal_css.resources)
    custom_id = 'ploneCustom.css'
    if custom_id in [res.getId() for res in resources]:
        portal_css.moveResource(custom_id, len(resources))


def is_plone_app_multilingual_installed(request):
    return IPloneAppMultilingualInstalled.providedBy(request)


def get_geocoder():
    return GeoCoderUtility()


class GeoCoderUtility(object):
    """Override collective.geo.mapwidget to use Nominatim instead of google
    """
    implements(IGeoCoder)

    def retrieve(self, address=None, google_api=None, language=None):
        self.geocoder = geocoders.Nominatim(user_agent='cpskinapp')

        if not address:
            raise GeocoderQueryError
        return self.geocoder.geocode(address, exactly_one=False,
                                     language=language)



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from collective.sendinblue.browser.portlet import PortletSubscribeForm
from collective.sendinblue.browser.portlet import Renderer
from collective.sendinblue.interfaces import INewsletterSubscribe
from cpskin.locales import CPSkinMessageFactory as _
from plone.app.portlets.portlets import base
from plone import api
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget
from plone.z3cform import z2
from plone.z3cform.interfaces import IWrappedForm
from z3c.form import button
from z3c.form import field
from z3c.form.interfaces import IFormLayer
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.interface import Interface
from zope.interface import Invalid
from zope.interface import alsoProvides


class IGDPRSubscribe(Interface):

    legal_conditions = schema.Bool(
        title=_(u'I Accept Legal terms and conditions'),
        description=_(u''),
        required=True,
    )


class GDPRRenderer(Renderer):

    def update(self):
        base.Renderer.update(self)
        z2.switch_on(self, request_layer=IFormLayer)
        self.form = GDPRPortletSubscribeForm(aq_inner(self.context), self.request, self.data)
        portal_url = api.portal.get_navigation_root(self.context).absolute_url()
        gpdr_url = '/'.join([portal_url, 'gdpr-view'])
        msgid = _(
            u'see_legal_conditions',
            default=u'See <a href="${legal_conditions_url}">legal terms and conditions</a>.',  # NOQA
            mapping={u'legal_conditions_url': gpdr_url},
        )
        legal_description = self.context.translate(msgid)
        gdpr_field = self.form.fields.get('legal_conditions')
        gdpr_field.field.description = legal_description
        alsoProvides(self.form, IWrappedForm)
        self.form.update()


class GDPRPortletSubscribeForm(PortletSubscribeForm):
    base_fields = field.Fields(INewsletterSubscribe)
    gdpr_fields = field.Fields(IGDPRSubscribe)
    fields = base_fields + gdpr_fields
    ignoreContext = True
    fields['captcha'].widgetFactory = ReCaptchaFieldWidget

    @button.buttonAndHandler(_('Subscribe'), name='subscribe')
    def handle_subscribe(self, action):
        data, errors = self.extractData()
        if not data.get('legal_conditions'):
            raise WidgetActionExecutionError(
                'legal_conditions',
                Invalid(_(u'You need to accept our legal terms and conditions.'))
            )

        return super(GDPRPortletSubscribeForm, self).handle_subscribe(
            self,
            action,
        )



# -*- coding: utf-8 -*-
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from collective.contact.core.browser.address import get_address
from cpskin.core.browser.common import CommonView
from cpskin.core.browser.contactdetails import ContactDetailsView
from zope.component import getMultiAdapter


class PreviewItem(ContactDetailsView, CommonView):

    def address(self):
        dict_address = get_address(self.context)
        template_path = 'address.pt'
        template = ViewPageTemplateFile(template_path)
        return template(self, dict_address)

    def city(self):
        dict_address = get_address(self.context)
        city = dict_address.get('city')
        return city

    def scaled_image_url(self, field):
        obj = self.context
        directory = self.request.get('directory')
        scale = getattr(directory, 'organization_image_scale', 'mini')
        url = ''
        images = obj.restrictedTraverse('@@images')
        if getattr(obj, field, False):
            image = images.scale(field, scale=scale)
            if image:
                url = image.url
        return url

    def show_photos_previews(self):
        directory = self.context
        return getattr(directory, 'show_organization_images', False)

    def render_contact_photo_preview(self, obj):
        context = self.context
        request = self.request
        request['directory'] = context
        render_view = u'faceted-preview-contact-photos'
        view = getMultiAdapter((obj, request), name=render_view)
        return view and view() or ''

    def render_item_preview(self, obj):
        context = self.context
        request = self.request
        scale = getattr(context, 'collection_image_scale', 'thumb')
        request['scale'] = scale
        request['collection'] = context
        render_view = u'faceted-preview-item'
        view = getMultiAdapter((obj, request), name=render_view)
        return view and view() or ''



# -*- coding: utf-8 -*-
from cpskin.core.interfaces import IElectedContentForTopMenu
from plone import api
from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class TopMenuViewlet(common.ViewletBase):

    index = ViewPageTemplateFile('top_menu.pt')

    def menus(self):
        portal_catalog = api.portal.get_tool('portal_catalog')
        interface_name = IElectedContentForTopMenu.__identifier__
        query = {'object_provides': interface_name}
        results = portal_catalog.searchResults(query)
        results = [r for r in results if r.exclude_from_nav]
        menus = []
        for r in results:
            query_path = {'query': r.getPath(), 'depth': 1}
            query = {
                'path': query_path,
                'sort_on': 'getObjPositionInParent',
                'review_state': ('published_and_shown', )
            }
            sub_items = portal_catalog.searchResults(query)
            sub_items = [s for s in sub_items if not s.exclude_from_nav]
            menu_item = {
                'root': r,
                'submenu': sub_items,
            }
            menus.append(menu_item)
        return menus

    def show_lead_image(self):
        portal_registry = api.portal.get_tool('portal_registry')
        return portal_registry['cpskin.core.interfaces.ICPSkinSettings.show_leadimage_in_action_menu']  # noqa



# -*- coding: utf-8 -*-
from cpskin.core.browser.folderview import configure_folderviews
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from cpskin.core.utils import add_keyword
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getMultiAdapter

import unittest


class TestKeywords(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        configure_folderviews(self.portal)
        self.folder = api.content.create(self.portal, 'Folder', 'folder')

    def test_keyword_indexview_behavior(self):
        add_behavior(
            'Collection',
            'cpskin.core.behaviors.indexview.ICpskinIndexViewSettings')
        collection = api.content.create(
            container=self.folder,
            type='Collection',
            id='testcollection')
        self.assertEqual(getattr(collection, 'index_view_keywords'), None)
        collection.index_view_keywords = (u'homepage',)
        self.assertEqual(
            getattr(collection, 'index_view_keywords'), (u'homepage',))

    def test_folderview_without_keyword_homepage(self):
        add_behavior('News Item', 'cpskin.core.behaviors.metadata.IHiddenTags')
        view = getMultiAdapter((self.portal, self.request), name='folderview')
        result = view.getResults(self.portal.actualites.actualites)
        self.assertTrue(result is None)

        # Adding content for fill collection
        news = api.content.create(
            container=self.portal.actualites,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=news, transition='publish')
        result = view.getResults(self.portal.actualites.actualites)
        self.assertEqual(len(result.get('standard-results', 0)), 1)

        add_keyword(news, 'hiddenTags', ['homepage'])
        add_keyword(news, 'hiddenTags', ['non-homepage'])

        result = view.getResults(self.portal.actualites.actualites)
        self.assertEqual(len(result.get('standard-results', 0)), 1)

    def test_folderview_with_keyword_homepage(self):
        add_behavior(
            'Collection',
            'cpskin.core.behaviors.indexview.ICpskinIndexViewSettings')
        add_behavior('News Item', 'cpskin.core.behaviors.metadata.IHiddenTags')
        self.portal.actualites.actualites.index_view_keywords = (u'homepage',)
        view = getMultiAdapter((self.portal, self.request), name='folderview')
        news = api.content.create(
            container=self.portal.actualites,
            type='News Item',
            id='testnewsitem')
        api.content.transition(obj=news, transition='publish')
        result = view.getResults(self.portal.actualites.actualites)
        self.assertTrue(result is None)

        add_keyword(news, 'hiddenTags', ['non-homepage'])
        result = view.getResults(self.portal.actualites.actualites)
        self.assertTrue(result is None)

        add_keyword(news, 'hiddenTags', ['homepage'])
        result = view.getResults(self.portal.actualites.actualites)
        self.assertEqual(len(result.get('standard-results', 0)), 1)



# -*- coding: utf-8 -*-
from plone import api
from plone.app.layout.viewlets.common import LogoViewlet
from zExceptions import NotFound


class CPSkinLogoViewlet(LogoViewlet):

    def update(self):
        super(CPSkinLogoViewlet, self).update()
        logoName = 'cpskinlogo.png'
        logoTitle = self.portal_state.portal_title()
        try:
            logo_custom = self.context.restrictedTraverse(logoName)
        except (AttributeError, KeyError, NotFound):
            # If no custom logo found, super() will handle default one
            pass
        else:
            try:
                self.logo_tag = logo_custom.tag(title=logoTitle, alt=logoTitle)
            except:
                scales = api.content.get_view(
                    name='images',
                    context=logo_custom,
                    request=self.request)
                self.logo_tag = scales.tag(title=logoTitle, alt=logoTitle)



# -*- coding: utf-8 -*-
import random

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets.common import ViewletBase
from zope.component import getMultiAdapter

from cpskin.core.utils import has_crop


HAS_MINISITE = False
try:
    from cpskin.minisite.interfaces import IInMinisite
    from cpskin.minisite.interfaces import IInPortal
    HAS_MINISITE = True
except ImportError:
    pass


class CPSkinBannerViewlet(ViewletBase):
    render = ViewPageTemplateFile('banner.pt')

    def available(self):
        context = self.context
        banner_view = getMultiAdapter((context, self.request),
                                      name="banner_activation")
        return banner_view.is_enabled

    def isInMinisite(self):
        if not HAS_MINISITE:
            return False
        return (self.isInMinisiteMode() or self.isInPortalMode())

    def show_slogan(self):
        portal_registry = api.portal.get_tool('portal_registry')
        show_slogan = portal_registry[
            'cpskin.core.interfaces.ICPSkinSettings.show_slogan']
        return show_slogan

    def get_slogan(self):
        default = {
            'title': '',
            'description': '',
        }
        if not self.isFolderView():
            return default
        banner = self.getBanner()
        if not banner['object']:
            return default
        obj = banner['object']
        return {
            'title': obj.Title(),
            'description': obj.Description(),
        }

    def homeUrl(self):
        """
        Returns URL of :
         - object where banner was activated if not in a minisite
         - minisite root object if in a minisite
        """
        context = self.context
        request = self.request
        if not HAS_MINISITE or not self.isInMinisite():
            banner_view = getMultiAdapter((context, request),
                                          name="banner_activation")
            return banner_view.banner_root.absolute_url()
        else:
            portal = api.portal.get()
            minisite = request.get('cpskin_minisite', None)
            minisiteRoot = portal.unrestrictedTraverse(minisite.search_path)
            return minisiteRoot.absolute_url()

    def isHomepage(self):
        if self.isInMinisite():
            return False
        obj = aq_inner(self.context)
        return INavigationRoot.providedBy(obj)

    def isFolderView(self):
        context = self.context
        layout = context.getLayout()
        return (layout == 'folderview')

    def isInMinisiteMode(self):
        if not HAS_MINISITE:
            return False
        request = self.request
        return IInMinisite.providedBy(request)

    def isInPortalMode(self):
        if not HAS_MINISITE:
            return False
        request = self.request
        return IInPortal.providedBy(request)

    def getUnscaleUrl(self, obj, fieldname, scale):
        if obj is None:
            return ''
        if has_crop(obj, fieldname, scale):
            view = obj.restrictedTraverse('@@images')
            scale = view.scale(fieldname, scale=scale)
            banner_url = scale.absolute_url()
        else:
            banner_url = '{0}/@@images/{1}'.format(
                obj.absolute_url(),
                fieldname,
            )
        return banner_url

    def getBanner(self):
        """
        Find and returns image or video banner informations.
        Banners types are searched by priotity :
         1. Event banner if context is event and image_banner field is filled
         2. Video (1) or random image (2) in local banner folder
         3. Local banner image
         4. Video (1) or random image (2) coming from inherited banner folder
         5. Inherited banner image
        Videos need 2 files (formats .mp4 & .webm) in the banner folder.
        """
        context = self.context
        local_banner_folder = getattr(context.aq_explicit, 'banner', None)
        local_banner_event = getattr(context.aq_explicit, 'image_banner', None)
        local_banner = getattr(context.aq_explicit, 'banner.jpg', None)
        banner_folder = getattr(context, 'banner', None)
        banner = getattr(context, 'banner.jpg', None)
        if context.portal_type == 'Event' and local_banner_event:
            return {
                'object': context,
                'url': self.getUnscaleUrl(context, 'image_banner', 'banner'),
                'type': 'image',
            }
        if local_banner_folder or (banner_folder and not local_banner):
            banner_folder_to_use = local_banner_folder and local_banner_folder or banner_folder
            brains = api.content.find(
                context=banner_folder_to_use,
                portal_type='File',
            )
            if brains:
                result = {
                    'object': None,
                    'type': 'video'
                }
                for brain in brains:
                    obj = brain.getObject()
                    content_type = obj.file.contentType
                    if content_type == 'video/mp4':
                        result['url'] = obj.absolute_url()
                    elif content_type == 'video/webm':
                        result['url_webm'] = obj.absolute_url()
                if result.get('url') and result.get('url_webm'):
                    return result
            brains = api.content.find(
                context=banner_folder_to_use,
                portal_type='Image',
            )
            if brains:
                brain = brains[random.randrange(len(brains))]
                obj = brain.getObject()
                return {
                    'object': obj,
                    'url': self.getUnscaleUrl(obj, 'image', 'banner'),
                    'type': 'image',
                }
        return {
            'object': banner,
            'url': self.getUnscaleUrl(banner, 'image', 'banner'),
            'type': 'image',
        }



# -*- coding: utf-8 -*-
from collective import dexteritytextindexer
from collective.z3cform.keywordwidget.field import Keywords
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.directives import form
from plone.formwidget.contenttree import ObjPathSourceBinder
from plone.supermodel import model
from z3c.form.browser.orderedselect import OrderedSelectFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import alsoProvides
from zope.interface import provider


class IStandardTags(model.Schema):
    model.fieldset(
        'categorization',
        label=_(u'label_schema_categorization', default=u'Categorization'),
        fields=('standardTags',),
    )

    form.widget(
        standardTags='collective.z3cform.keywordwidget.widget.KeywordFieldWidget')  # noqa
    standardTags = Keywords(
        title=_(u'label_standardTags', default=u'Standard Tags'),
        description=_(
            u'help_standard_tags',
            default=u'Standard Tags are used for webmaster '
                    u'organization of content.',
        ),
        required=False,
        # Automatically get the index in catalog by name
        index_name='standardTags',
    )


class IHiddenTags(model.Schema):
    model.fieldset(
        'categorization',
        label=_(u'label_schema_categorization', default=u'Categorization'),
        fields=('hiddenTags',),
    )

    form.widget(
        hiddenTags='collective.z3cform.keywordwidget.widget.KeywordFieldWidget')  # noqa
    hiddenTags = Keywords(
        title=_(u'label_hidden_tags', default=u'Hidden Tags'),
        description=_(
            u'help_hidden_tags',
            default=u'Hidden Tags are used for webmaster '
                    u'organization of content.',
        ),
        required=False,
        # Automatically get the index in catalog by name
        index_name='hiddenTags',
    )


class IISearchTags(model.Schema):
    model.fieldset(
        'categorization',
        label=_(u'label_schema_categorization', default=u'Categorization'),
        fields=('isearchTags',),
    )

    form.widget(
        isearchTags='collective.z3cform.keywordwidget.widget.KeywordFieldWidget')  # noqa
    isearchTags = Keywords(
        title=_(u'label_isearch_tags', default=u'I Search Tags'),
        description=_(
            u'help_isearch_tags',
            default=u'I Search Tags are used for webmaster '
                    u'organization of content.'
        ),
        required=False,
        index_name='isearchTags',
    )


class IIAmTags(model.Schema):
    model.fieldset(
        'categorization',
        label=_(u'label_schema_categorization', default=u'Categorization'),
        fields=('iamTags',),
    )

    form.widget(
        iamTags='collective.z3cform.keywordwidget.widget.KeywordFieldWidget')
    iamTags = Keywords(
        title=_(u'label_iam_tags', default=u'I Am Tags'),
        description=_(
            u'help_iam_tags',
            default=u'I Am Tags are used for webmaster '
                    u'organization of content.',
        ),
        required=False,
        # Automatically get the index in catalog by name
        index_name='iamTags',
    )


alsoProvides(IStandardTags, IFormFieldProvider)
alsoProvides(IHiddenTags, IFormFieldProvider)
alsoProvides(IISearchTags, IFormFieldProvider)
alsoProvides(IIAmTags, IFormFieldProvider)

# This is an old interface but there are always some objects that were registered with it (in Florenne) so when we want to register some new contact (with linked contact), sometimes that crash.
# SUP-7077
class IUseKeywordHomepage(model.Schema):
    pass


@provider(IFormFieldProvider)
class IRelatedContacts(model.Schema):
    model.fieldset(
        'related_contacts',
        label=_(u'Related contacts'),
        fields=('aboveContentContact', 'aboveVisbileFields',
                'belowContentContact', 'belowVisbileFields',
                'see_map', 'see_logo_in_popup'),
    )

    aboveContentContact = RelationList(
        title=u'Above content related contact',
        default=[],
        value_type=RelationChoice(
            title=_(u'Related'),
            source=ObjPathSourceBinder(
                portal_type=('person', 'organization', 'held_position')
            )
        ),
        required=False,
    )

    form.widget(aboveVisbileFields=OrderedSelectFieldWidget)
    aboveVisbileFields = schema.Tuple(
        title=_(u'Visible fields for above viewlet'),
        description=_(u'Please select which fields should be visible.'),
        required=False,
        default=('street', 'number', 'zip_code', 'city', 'phone', 'cell_phone',
                 'fax', 'email', 'website', 'activity'),
        value_type=schema.Choice(
            vocabulary=u'cpskin.core.vocabularies.contact_fields'
        )
    )

    belowContentContact = RelationList(
        title=u'Below content related contact',
        default=[],
        value_type=RelationChoice(
            title=_(u'Related'),
            source=ObjPathSourceBinder(
                portal_type=('person', 'organization', 'held_position')
            )
        ),
        required=False,
    )

    form.widget(belowVisbileFields=OrderedSelectFieldWidget)
    belowVisbileFields = schema.Tuple(
        title=_(u'Visible fields for below viewlet'),
        description=_(u'Please select which fields should be visible.'),
        required=False,
        default=('title',),
        value_type=schema.Choice(
            vocabulary=u'cpskin.core.vocabularies.contact_fields'
        )
    )

    see_map = schema.Bool(
        title=_(u'See map'),
        description=_(u'Do you want to map with links of contacts ?'),
        required=False,
        default=True
    )

    see_logo_in_popup = schema.Bool(
        title=_(u'See logo on map popup'),
        description=_(u'Do you want to logo of contact on the popup on map ?'),
        required=False,
        default=True
    )


@provider(IFormFieldProvider)
class IAdditionalSearchableText(model.Schema):
    model.fieldset(
        'categorization',
        label=_(u'label_schema_categorization', default=u'Categorization'),
        fields=('additional_searchable_text',),
    )
    dexteritytextindexer.searchable('additional_searchable_text')
    additional_searchable_text = schema.Text(
        title=_(u'Additional searchable text'),
        required=False)
